//==========================================================
//	ARTRAY Camera / Capture Module Software Developer Kit
//
//						Version 1.20		2003.9.4
//						Version 1.21		2003.9.18
//						Version 1.22		2003.10.10
//						Version 1.23		2004.1.21
//						Version 1.235		2004.3.19
//						Version 1.24		2004.7.9
//						Version 1.245		2004.8.17
//						Version 1.25		2004.9.1
//						Version 1.26		2004.10.28
//						Version 1.27		2005.3.8
//						Version 1.275		2005.5.23
//						Version 1.276		2005.9.5
//						Version 1.277		2005.12.22
//
//									(C) 2002-2005 Artray
//==========================================================


#ifndef _ARTCAM_SDK_
#define _ARTCAM_SDK_

#include <windows.h>



#define WM_GRAPHNOTIFY  WM_APP + 1		// It is not used in this version
#define WM_GRAPHPAINT	WM_APP + 2		// When an image is updated, it is sent to application
#define WM_ERROR		WM_APP + 3		// When an error occurs, it is sent to application


#define  HACAM  DWORD

enum {
	ARTCAM_CAMERATYPE_DS		= 1,	// DirectShow Camera
	ARTCAM_CAMERATYPE_DISABLE1	= 2,	// Disable
	ARTCAM_CAMERATYPE_USTC		= 3,	// ARTUST
	ARTCAM_CAMERATYPE_CNV		= 4,	// ARTCNV
	ARTCAM_CAMERATYPE_DISABLE2	= 5,	// Disable
	ARTCAM_CAMERATYPE_130MI		= 6,	// ARTCAM-130MI
	ARTCAM_CAMERATYPE_DISABLE3	= 7,	// Disable
	ARTCAM_CAMERATYPE_200MI		= 8,	// ARTCAM-200MI
	ARTCAM_CAMERATYPE_300MI		= 9,	// ARTCAM-300MI
	ARTCAM_CAMERATYPE_150P		= 10,	// ARTCAM-150P
	ARTCAM_CAMERATYPE_320P		= 11,	// ARTCAM-320P
	ARTCAM_CAMERATYPE_DISABLE4	= 12,	// Disable
	ARTCAM_CAMERATYPE_200SH		= 13,	// ARTCAM-200SH
	ARTCAM_CAMERATYPE_098		= 14,	// ARTCAM-098
	ARTCAM_CAMERATYPE_036MI		= 15,	// ARTCAM-036MI
	ARTCAM_CAMERATYPE_500P		= 16,	// ARTCAM-500P
	ARTCAM_CAMERATYPE_150P2		= 17,	// ARTCAM-150PII
};

// Error code
enum {
	ARTCAMSDK_NOERROR = 0,			// Normal
	ARTCAMSDK_NOT_INITIALIZE,		// It is initialized and does not become precocious
	ARTCAMSDK_DISABLEDDEVICE,		// Not initialised
	ARTCAMSDK_CREATETHREAD,			// The thread creation for picture taking in goes wrong
	ARTCAMSDK_CREATEWINDOW,			// Window creation goes wrong
	ARTCAMSDK_OUTOFMEMORY,			// sufficient memory for image transmission is not secured   or --   Reservation of a memory goes wrong
	ARTCAMSDK_CAMERASET,			// An error is made in a setup of a camera (device)
	ARTCAMSDK_CAMERASIZE,			// An error is made in a size setup of a camera (device)
	ARTCAMSDK_CAPTURE,				// Image CAPTURE in goes wrong
	ARTCAMSDK_PARAM,				// The argument is wrong
	ARTCAMSDK_DIRECTSHOW,			// DirectShow initialization error
	ARTCAMSDK_UNSUPPORTED,			// This function is not supported
	ARTCAMSDK_UNKNOWN,				// An unknown error
	ARTCAMSDK_CAPTURELOST,			// A device disappears
};

// Information to transmit

typedef struct GP_INFO {
	LONG	lSize;
	LONG	lWidth;
	LONG	lHeight;
	LONG	lBpp;
	LONG	lFps;
} *LPGP_INFO;


// Classification of a picture filter
enum {
	ARTCAM_FILTERTYPE_RESERVE = 0,			// It is reserved
	ARTCAM_FILTERTYPE_BRIGHTNESS,			// Brightness
	ARTCAM_FILTERTYPE_CONTRAST,				// Contrast
	ARTCAM_FILTERTYPE_HUE,					// HUE
	ARTCAM_FILTERTYPE_SATURATION,			// SATURATION
	ARTCAM_FILTERTYPE_SHARPNESS,			// SHARPNESS
	ARTCAM_FILTERTYPE_BAYER_GAIN_RGB,		// The color gain of bayer conversion
	ARTCAM_FILTERTYPE_BAYER_GAIN_R,			// The color gain of bayer conversion(Only red)
	ARTCAM_FILTERTYPE_BAYER_GAIN_G,			// The color gain of bayer conversion(Only green)
	ARTCAM_FILTERTYPE_BAYER_GAIN_B,			// The color gain of bayer conversion(Only blue)
	ARTCAM_FILTERTYPE_BAYER_GAIN_AUTO,		// Automatic white balance adjustment
	ARTCAM_FILTERTYPE_GAMMA,				// Gamma
	ARTCAM_FILTERTYPE_BAYERMODE,			// bayer conversion mode
	ARTCAM_FILTERTYPE_GLOBAL_GAIN,			// The global gain of a camera
	ARTCAM_FILTERTYPE_COLOR_GAIN_R,			// The color gain of a camera (red)
	ARTCAM_FILTERTYPE_COLOR_GAIN_G1,		// The color gain of a camera (green 1)
	ARTCAM_FILTERTYPE_COLOR_GAIN_G2,		// The color gain of a camera (green 2)
	ARTCAM_FILTERTYPE_COLOR_GAIN_B,			// The color gain of a camera (blue)
	ARTCAM_FILTERTYPE_EXPOSURETIME,			// Exposure time (shutter speed)
};

// Compatibility with the past version
#define ARTCAM_FILTERTYPE_RGB_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_RGB
#define ARTCAM_FILTERTYPE_R_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_R
#define ARTCAM_FILTERTYPE_G_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_G
#define ARTCAM_FILTERTYPE_B_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_B
#define ARTCAM_FILTERTYPE_AUTO_GAIN		ARTCAM_FILTERTYPE_BAYER_GAIN_AUTO


// Infanticide transmission mode 
enum {
	SUBSAMPLE_1 = 0,	// Full size
	SUBSAMPLE_2,		// 1/2
	SUBSAMPLE_4,		// 1/4
	SUBSAMPLE_8,		// 1/8
};


// Auto iris mode
enum {
	AI_NONE = 0,		// Invalid
	AI_EXPOSURE,		// Exposure time
	AI_GAIN,			// Gain
};


// Video format
enum {
	VIDEOFORMAT_NTSC = 0,	// NTSC
	VIDEOFORMAT_PAL,		// PAL
	VIDEOFORMAT_PALM,		// PALM
	VIDEOFORMAT_SECAM,		// SECAM
};

// Sampling rate
enum {
	WIDE_HISPEED = 0,	// 
	WIDE_LOWSPEED,		// 
	NORMAL_HISPEED,		// 
	NORMAL_LOWSPEED,	// 
};


// Picture preservation form
enum {
	FILETYPE_BITMAP = 0,	// Bitmap saving (large size).
	FILETYPE_RAW,			// Only data is saved (large size)
	FILETYPE_JPEG_HIGH,		// Save in high-quality JPEG format (medium size). Characters and lines are distorted.
	FILETYPE_JPEG_NOMAL,	// Save in standard JPEG format (small size). Small patterns are distorted.
	FILETYPE_JPEG_LOW,		// Save in low-quality JPEG (small size). Block noise are seen.
	FILETYPE_PNG,			// Save as PNG. 16 bits image can be saved correctly. Size=Medium
	FILETYPE_TIFF,			// Save as TIFF. 16 bits image can be saved correctly. Size=Large
};


/*
 *Remark*
 16 bit image cannot be read in every image-processing software or in every environment.
 16 bit PNG and TIFF saved by this SDK can be read by our Veiwer Software Version2 or later and Adobe Photoshop6.
 For all other software, deterioration of image or an error may be experienced.
 Since it saves as a 16-bit gray scale picture when saved by 16-bit bitmap   The color palette etc. is not saved. 
 Since it saves as a 16-bit gray scale picture when saved by 16-bit bitmap   The color palette etc. is not saved.
 
 In case it saves by 16 bits which needs to prepare a 16-bit palette by the software side, 
 or needs to prepare the device context of gray scale, we recommend you to save by TIFF or PNG. 
 In JPEG, it cannot save other than a color picture. 
 Especially about a 16-bit picture, data of 8 bits of low ranks is omitted and saved. 
*/





#ifdef __cplusplus
extern "C"{
#endif

	DWORD		ArtCam_GetDllVersion		(void);
	LONG		ArtCam_GetLastError			(HACAM hACam);

	HACAM		ArtCam_Initialize			(HWND hWnd);
	BOOL		ArtCam_Release				(HACAM hACam);

	BOOL		ArtCam_Preview				(HACAM hACam);
	BOOL		ArtCam_Record				(HACAM hACam, LPCTSTR lpAviName, UINT RecTime, BOOL fShow);
	BOOL		ArtCam_CallBackPreview		(HACAM hACam, HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown);
	BOOL		ArtCam_Close				(HACAM hACam);

	// 1230
	BOOL		ArtCam_SnapShot				(HACAM hACam, LPBYTE lpImage, LONG Size, BOOL TopDown);

	// 1250
	BOOL		ArtCam_Capture				(HACAM hACam);

	// 1260
	BOOL		ArtCam_Trigger				(HACAM hACam, HWND hWnd, LPBYTE lpImage, LONG Size, BOOL TopDown);
	BOOL		ArtCam_SaveImage			(HACAM hACam, LPCTSTR lpSaveName, LONG FileType);


	BOOL		ArtCam_SetPreviewWindow		(HACAM hACam, HWND hWnd, LONG Left, LONG Top, LONG Right, LONG Bottom);
	BOOL		ArtCam_SetCaptureWindow		(HACAM hACam, LONG Width, LONG Height, LONG Fps);
	BOOL		ArtCam_SetCaptureWindowEx	(HACAM hACam, LONG HTotal, LONG HStart, LONG HEffective, LONG VTotal, LONG VStart, LONG VEffective);
	BOOL		ArtCam_GetCaptureWindowEx	(HACAM hACam, LONG* HTotal, LONG* HStart, LONG* HEffective, LONG* VTotal, LONG* VStart, LONG* VEffective);

	BOOL		ArtCam_SetColorMode			(HACAM hACam, LONG ColorMode);
	LONG		ArtCam_GetColorMode			(HACAM hACam);

	BOOL		ArtCam_SetCrossbar			(HACAM hACam, LONG Output, LONG Input);
	BOOL		ArtCam_SetDeviceNumber		(HACAM hACam, LONG Number);
	LONG		ArtCam_EnumDevice			(HACAM hACam, TCHAR szDeviceName[10][256]);
	BOOL		ArtCam_GetDeviceName		(HACAM hACam, LONG index, LPSTR szDeviceName, LONG nSize);

	BOOL		ArtCam_StartPreview			(HACAM hACam);
	BOOL		ArtCam_StopPreview			(HACAM hACam);
	BOOL		ArtCam_GetImage				(HACAM hACam, LPBYTE lpImage, LONG Size, BOOL TopDown);

	LONG		ArtCam_Width				(HACAM hACam);
	LONG		ArtCam_Height				(HACAM hACam);
	LONG		ArtCam_Fps					(HACAM hACam);

	BOOL		ArtCam_SetCameraDlg			(HACAM hACam, HWND hWnd);
	BOOL		ArtCam_SetImageDlg			(HACAM hACam, HWND hWnd);
	BOOL		ArtCam_SetAnalogDlg			(HACAM hACam, HWND hWnd);

	// 1230
	BOOL		ArtCam_SetIOPort			(HACAM hACam, BYTE byteData, LONG longData, DWORD Reserve);
	BOOL		ArtCam_GetIOPort			(HACAM hACam, LPBYTE byteData, LPLONG longData, DWORD Reserve);

	BOOL		ArtCam_SetFilterValue		(HACAM hACam, LONG FilterType, LONG Value);
	LONG		ArtCam_GetFilterValue		(HACAM hACam, LONG FilterType, LPBOOL Error);

	BOOL		ArtCam_SetSubSample			(HACAM hACam, LONG SubSampleMode);
	LONG		ArtCam_GetSubSample			(HACAM hACam);

	BOOL		ArtCam_SetWaitTime			(HACAM hACam, LONG WaitTime);
	LONG		ArtCam_GetWaitTime			(HACAM hACam);

	// 1240
	BOOL		ArtCam_SetMirrorV			(HACAM hACam, BOOL Flg);
	BOOL		ArtCam_GetMirrorV			(HACAM hACam);

	BOOL		ArtCam_SetMirrorH			(HACAM hACam, BOOL Flg);
	BOOL		ArtCam_GetMirrorH			(HACAM hACam);

	BOOL		ArtCam_SetBrightness		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetBrightness		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetContrast			(HACAM hACam, LONG Value);
	LONG		ArtCam_GetContrast			(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetHue				(HACAM hACam, LONG Value);
	LONG		ArtCam_GetHue				(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetSaturation		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetSaturation		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetSharpness			(HACAM hACam, LONG Value);
	LONG		ArtCam_GetSharpness			(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetBayerGainRGB		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetBayerGainRGB		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetBayerGainRed		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetBayerGainRed		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetBayerGainGreen	(HACAM hACam, LONG Value);
	LONG		ArtCam_GetBayerGainGreen	(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetBayerGainBlue		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetBayerGainBlue		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetBayerGainAuto		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetBayerGainAuto		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetGamma				(HACAM hACam, LONG Value);
	LONG		ArtCam_GetGamma				(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetBayerMode			(HACAM hACam, LONG Value);
	LONG		ArtCam_GetBayerMode			(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetGlobalGain		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetGlobalGain		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetColorGainRed		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetColorGainRed		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetColorGainGreen1	(HACAM hACam, LONG Value);
	LONG		ArtCam_GetColorGainGreen1	(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetColorGainGreen2	(HACAM hACam, LONG Value);
	LONG		ArtCam_GetColorGainGreen2	(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetColorGainBlue		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetColorGainBlue		(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetExposureTime		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetExposureTime		(HACAM hACam, LPBOOL Error);

	// 1260
	BOOL		ArtCam_SetHalfClock			(HACAM hACam, LONG Value);
	LONG		ArtCam_GetHalfClock			(HACAM hACam, LPBOOL Error);

	BOOL		ArtCam_SetAutoIris			(HACAM hACam, LONG Value);
	LONG		ArtCam_GetAutoIris			(HACAM hACam, LPBOOL Error);

	// 1275
	BOOL		ArtCam_SetSamplingRate		(HACAM hACam, LONG Value);
	LONG		ArtCam_GetSamplingRate		(HACAM hACam, LPBOOL Error);

	LONG		ArtCam_GetVideoFormat		(HACAM hACam, LPBOOL Error);

	// 1276
	BOOL		ArtCam_WriteSromID			(HACAM hACam, LONG Address, LONG Value);
	LONG		ArtCam_ReadSromID			(HACAM hACam, LONG Address, LPBOOL Error);


#ifdef __cplusplus
}			// extern "C"{
#endif

#endif		// #ifndef _ARTCAM_SDK_




