'==========================================================
'	ARTRAY Camera / Capture Module Software Developer Kit
'
'						Version 1.20		2003.9.4
'						Version 1.21		2003.9.18
'						Version 1.22		2003.10.10
'						Version 1.23		2004.1.21
'						Version 1.235		2004.3.19
'						Version 1.24		2004.7.9
'						Version 1.245		2004.8.17
'						Version 1.25		2004.9.1
'						Version 1.26		2004.10.28
'						Version 1.27		2005.3.8
'						Version 1.275		2005.5.23
'						Version 1.276		2005.6.27
'						Version 1.277		2005.12.22
'
'									(C) 2002-2005 Artray
'==========================================================

Namespace ArtCamSdk

    Public Enum DLL_MESSAGE
        WM_GRAPHNOTIFY = &H8001  ' Not used in this version.
        WM_GRAPHPAINT            ' Sent to application when an image is updated.
        WM_ERROR                 ' Sent to application when an error occurs
    End Enum

    Public Enum ARTCAM_CAMERATYPE
        ARTCAM_CAMERATYPE_DS = 1    ' DirectShow Camera
        ARTCAM_CAMERATYPE_DISABLE1  ' Invalid
        ARTCAM_CAMERATYPE_USTC      ' ARTUST-0209C
        ARTCAM_CAMERATYPE_CNV       ' ARTCNV
        ARTCAM_CAMERATYPE_DISABLE2  ' Invalid
        ARTCAM_CAMERATYPE_130MI     ' ARTCAM-130MI
        ARTCAM_CAMERATYPE_DISABLE3  ' Invalid
        ARTCAM_CAMERATYPE_200MI     ' ARTCAM-200MI
        ARTCAM_CAMERATYPE_300MI     ' ARTCAM-300MI
        ARTCAM_CAMERATYPE_150P      ' ARTCAM-150P
        ARTCAM_CAMERATYPE_320P      ' ARTCAM-320P
        ARTCAM_CAMERATYPE_DISABLE4  ' Invalid
        ARTCAM_CAMERATYPE_200SH     ' ARTCAM-200SH
        ARTCAM_CAMERATYPE_098       ' ARTCAM-098
        ARTCAM_CAMERATYPE_036MI     ' ARTCAM-036MI
        ARTCAM_CAMERATYPE_500P      ' ARTCAM-500P
        ARTCAM_CAMERATYPE_150P2     ' ARTCAM-150P2
    End Enum

    ' Error Code
    Public Enum ARTCAMSDK_ERROR
        ARTCAMSDK_NOERROR = 0      ' Normal
        ARTCAMSDK_NOT_INITIALIZE   ' Not initialized
        ARTCAMSDK_DISABLEDDEVICE   ' Access to unavailable device was attempted
        ARTCAMSDK_CREATETHREAD     ' Failure of thread creation for capturing
        ARTCAMSDK_CREATEWINDOW     ' Window creation failed
        ARTCAMSDK_OUTOFMEMORY      ' Not enough memory for image transfer/Failure for securing memory
        ARTCAMSDK_CAMERASET        ' Error for camera (device) settings
        ARTCAMSDK_CAMERASIZE       ' Error for camera (device) size settings
        ARTCAMSDK_CAPTURE          ' Capturing failed
        ARTCAMSDK_PARAM            ' Wrong argument
        ARTCAMSDK_DIRECTSHOW       ' Directshow initialization error
        ARTCAMSDK_UNSUPPORTED      ' Not supported
        ARTCAMSDK_UNKNOWN          ' Unknow error
        ARTCAMSDK_CAPTURELOST      ' Device lost
    End Enum

    ' Type of image filter
    Public Enum ARTCAM_FILTERTYPE
        ARTCAM_FILTERTYPE_RESERVE = 0       ' Reserved
        ARTCAM_FILTERTYPE_BRIGHTNESS        ' Brightness
        ARTCAM_FILTERTYPE_CONTRAST          ' Contrast
        ARTCAM_FILTERTYPE_HUE               ' Hue
        ARTCAM_FILTERTYPE_SATURATION        ' Saturation
        ARTCAM_FILTERTYPE_SHARPNESS         ' Sharpness
        ARTCAM_FILTERTYPE_BAYER_GAIN_RGB    ' Bayer coversion color gain
        ARTCAM_FILTERTYPE_BAYER_GAIN_R      ' Color gain (red only) for bayer conversion
        ARTCAM_FILTERTYPE_BAYER_GAIN_G      ' Color gain (green only) for bayer conversion
        ARTCAM_FILTERTYPE_BAYER_GAIN_B      ' Color gain (Blue only) for bayer conversion
        ARTCAM_FILTERTYPE_BAYER_GAIN_AUTO   ' Auto white balance
        ARTCAM_FILTERTYPE_GAMMA             ' Gumma
        ARTCAM_FILTERTYPE_BAYERMODE         ' Bayer conversion mode
        ARTCAM_FILTERTYPE_GLOBAL_GAIN       ' Global gain for camera
        ARTCAM_FILTERTYPE_COLOR_GAIN_R      ' Color gain (red) for camera
        ARTCAM_FILTERTYPE_COLOR_GAIN_G1     ' Color gain (green1) for camera
        ARTCAM_FILTERTYPE_COLOR_GAIN_G2     ' Color gain (green2) for camera
        ARTCAM_FILTERTYPE_COLOR_GAIN_B      ' Color gain (blue) for camera
        ARTCAM_FILTERTYPE_EXPOSURETIME      ' Exposure time (shutter speed)
    End Enum

    ' Pixel skipping transfer mode
    Public Enum SUBSAMPLE
        SUBSAMPLE_1 = 0  ' Full size
        SUBSAMPLE_2      ' 1/2
        SUBSAMPLE_4      ' 1/4
        SUBSAMPLE_8      ' 1/8
    End Enum

    ' Auto iris mode
    Public Enum AI_TYPE
        AI_NONE = 0        ' Invalid
        AI_EXPOSURE        ' Exposure time
        AI_GAIN            ' Gain
    End Enum

    ' Video format
    Public Enum VIDEOFORMAT
        VIDEOFORMAT_NTSC = 0    ' NTSC
        VIDEOFORMAT_PAL         ' PAL
        VIDEOFORMAT_PALM        ' PALM
        VIDEOFORMAT_SECAM       ' SECAM
    End Enum

    ' Sampling rate
    Public Enum SAMPLING_RATE
        WIDE_HISPEED = 0    ' 
        WIDE_LOWSPEED       ' 
        NORMAL_HISPEED      ' 
        NORMAL_LOWSPEED     ' 
    End Enum

    ' Format for image saving
    Public Enum FILETYPE
        FILETYPE_BITMAP = 0
        FILETYPE_RAW
        FILETYPE_JPEG_HIGH
        FILETYPE_JPEG_NOMAL
        FILETYPE_JPEG_LOW
        FILETYPE_PNG
        FILETYPE_TIFF
    End Enum


    Public Class ArtCam

        Declare Function GetDllVersion Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetDllVersion" () As Integer
        Declare Function GetLastError Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetLastError" (ByVal hACam As Integer) As Integer

        Declare Function Initialize Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Initialize" (ByVal hWnd As IntPtr) As Integer
        Declare Function Release Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Release" (ByVal hACam As Integer) As Integer

        Declare Function Preview Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Preview" (ByVal hACam As Integer) As Integer
        Declare Function Record Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Record" (ByVal hACam As Integer, ByVal lpAviName As String, ByVal RecTime As Integer, ByVal fShow As Integer) As Integer
        Declare Function CallBackPreviewA Lib "ArtCamSdk_320P.dll" Alias "ArtCam_CallBackPreview" (ByVal hACam As Integer, ByVal hWnd As IntPtr, ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
        Declare Function CallBackPreviewP Lib "ArtCamSdk_320P.dll" Alias "ArtCam_CallBackPreview" (ByVal hACam As Integer, ByVal hWnd As IntPtr, ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer
        Declare Function SnapShotA Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SnapShot" (ByVal hACam As Integer, ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
        Declare Function SnapShotP Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SnapShot" (ByVal hACam As Integer, ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer
        Declare Function Close Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Close" (ByVal hACam As Integer) As Integer

        ' 1250
        Declare Function Capture Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Capture" (ByVal hACam As Integer) As Integer

        ' 1260
        Declare Function TriggerA Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Trigger" (ByVal hACam As Integer, ByVal hWnd As IntPtr, ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
        Declare Function TriggerP Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Trigger" (ByVal hACam As Integer, ByVal hWnd As IntPtr, ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer
        Declare Function SaveImage Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SaveImage" (ByVal hACam As Integer, ByVal lpSaveName As String, ByVal FileType As FILETYPE) As Integer

        Declare Function SetPreviewWindow Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetPreviewWindow" (ByVal hACam As Integer, ByVal hWnd As IntPtr, ByVal Left As Integer, ByVal Top As Integer, ByVal Right As Integer, ByVal Bottom As Integer) As Integer
        Declare Function SetCaptureWindow Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetCaptureWindow" (ByVal hACam As Integer, ByVal Width As Integer, ByVal Height As Integer, ByVal Frame As Integer) As Integer
        Declare Function SetCaptureWindowEx Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetCaptureWindowEx" (ByVal hACam As Integer, ByVal HTotal As Integer, ByVal HStart As Integer, ByVal HEffective As Integer, ByVal VTotal As Integer, ByVal VStart As Integer, ByVal VEffective As Integer) As Integer
        Declare Function GetCaptureWindowEx Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetCaptureWindowEx" (ByVal hACam As Integer, ByRef HTotal As Integer, ByRef HStart As Integer, ByRef HEffective As Integer, ByRef VTotal As Integer, ByRef VStart As Integer, ByRef VEffective As Integer) As Integer

        Declare Function SetColorMode Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetColorMode" (ByVal hACam As Integer, ByVal ColorMode As Integer) As Integer
        Declare Function GetColorMode Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetColorMode" (ByVal hACam As Integer) As Integer

        Declare Function SetCrossbar Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetCrossbar" (ByVal hACam As Integer, ByVal Output As Integer, ByVal Input As Integer) As Integer
        Declare Function SetDeviceNumber Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetDeviceNumber" (ByVal hACam As Integer, ByVal Number As Integer) As Integer
        Declare Function GetDeviceName Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetDeviceName" (ByVal hACam As Integer, ByVal index As Integer, ByVal szDeviceName As System.Text.StringBuilder, ByVal Size As Integer) As Integer

        Declare Function StartPreview Lib "ArtCamSdk_320P.dll" Alias "ArtCam_StartPreview" (ByVal hACam As Integer) As Integer
        Declare Function StopPreview Lib "ArtCamSdk_320P.dll" Alias "ArtCam_StopPreview" (ByVal hACam As Integer) As Integer
        Declare Function GetImageA Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetImage" (ByVal hACam As Integer, ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
        Declare Function GetImageP Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetImage" (ByVal hACam As Integer, ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer

        Declare Function Width Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Width" (ByVal hACam As Integer) As Integer
        Declare Function Height Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Height" (ByVal hACam As Integer) As Integer
        Declare Function Fps Lib "ArtCamSdk_320P.dll" Alias "ArtCam_Fps" (ByVal hACam As Integer) As Integer

        Declare Function SetCameraDlg Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetCameraDlg" (ByVal hACam As Integer, ByVal hWnd As IntPtr) As Integer
        Declare Function SetImageDlg Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetImageDlg" (ByVal hACam As Integer, ByVal hWnd As IntPtr) As Integer
        Declare Function SetAnalogDlg Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetAnalogDlg" (ByVal hACam As Integer, ByVal hWnd As IntPtr) As Integer

        ' 1230
        Declare Function SetIOPort Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetIOPort" (ByVal hACam As Integer, ByVal byteData As Integer, ByVal longData As Integer, ByVal Reserve As Integer) As Integer
        Declare Function GetIOPort Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetIOPort" (ByVal hACam As Integer, ByVal byteData As IntPtr, ByVal longData As IntPtr, ByVal Reserve As Integer) As Integer

        Declare Function SetFilterValue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetFilterValue" (ByVal hACam As Integer, ByVal FilterType As Integer, ByVal Value As Integer) As Integer
        Declare Function GetFilterValue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetFilterValue" (ByVal hACam As Integer, ByVal FilterType As Integer, ByRef pError As Integer) As Integer

        Declare Function SetSubSample Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetSubSample" (ByVal hACam As Integer, ByVal SubSampleMode As Integer) As Integer
        Declare Function GetSubSample Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetSubSample" (ByVal hACam As Integer) As Integer

        Declare Function SetWaitTime Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetWaitTime" (ByVal hACam As Integer, ByVal WaitTime As Integer) As Integer
        Declare Function GetWaitTime Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetWaitTime" (ByVal hACam As Integer) As Integer

        ' 1240
        Declare Function SetMirrorV Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetMirrorV" (ByVal hACam As Integer, ByVal Flg As Integer) As Integer
        Declare Function GetMirrorV Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetMirrorV" (ByVal hACam As Integer) As Integer

        Declare Function SetMirrorH Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetMirrorH" (ByVal hACam As Integer, ByVal Flg As Integer) As Integer
        Declare Function GetMirrorH Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetMirrorH" (ByVal hACam As Integer) As Integer

        Declare Function SetBrightness Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetBrightness" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetBrightness Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetBrightness" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetContrast Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetContrast" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetContrast Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetContrast" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetHue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetHue" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetHue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetHue" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetSaturation Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetSaturation" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetSaturation Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetSaturation" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetSharpness Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetSharpness" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetSharpness Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetSharpness" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetBayerGainRGB Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetBayerGainRGB" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetBayerGainRGB Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetBayerGainRGB" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetBayerGainRed Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetBayerGainRed" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetBayerGainRed Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetBayerGainRed" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetBayerGainGreen Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetBayerGainGreen" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetBayerGainGreen Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetBayerGainGreen" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetBayerGainBlue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetBayerGainBlue" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetBayerGainBlue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetBayerGainBlue" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetBayerGainAuto Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetBayerGainAuto" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetBayerGainAuto Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetBayerGainAuto" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetGamma Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetGamma" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetGamma Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetGamma" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetBayerMode Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetBayerMode" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetBayerMode Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetBayerMode" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetGlobalGain Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetGlobalGain" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetGlobalGain Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetGlobalGain" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetColorGainRed Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetColorGainRed" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetColorGainRed Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetColorGainRed" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetColorGainGreen1 Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetColorGainGreen1" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetColorGainGreen1 Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetColorGainGreen1" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetColorGainGreen2 Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetColorGainGreen2" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetColorGainGreen2 Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetColorGainGreen2" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetColorGainBlue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetColorGainBlue" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetColorGainBlue Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetColorGainBlue" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetExposureTime Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetExposureTime" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetExposureTime Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetExposureTime" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        ' 1260
        Declare Function SetHalfClock Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetHalfClock" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetHalfClock Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetHalfClock" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        Declare Function SetAutoIris Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetAutoIris" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetAutoIris Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetAutoIris" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        ' 1275
        Declare Function SetSamplingRate Lib "ArtCamSdk_320P.dll" Alias "ArtCam_SetSamplingRate" (ByVal hACam As Integer, ByVal Value As Integer) As Integer
        Declare Function GetSamplingRate Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetSamplingRate" (ByVal hACam As Integer, ByRef pError As Integer) As Integer
        Declare Function GetVideoFormat Lib "ArtCamSdk_320P.dll" Alias "ArtCam_GetVideoFormat" (ByVal hACam As Integer, ByRef pError As Integer) As Integer

        ' 1276
        Declare Function WriteSromID Lib "ArtCamSdk_320P.dll" Alias "ArtCam_WriteSromID" (ByVal hACam As Integer, ByVal Address As Integer, ByVal Value As Integer) As Integer
        Declare Function ReadSromID Lib "ArtCamSdk_320P.dll" Alias "ArtCam_ReadSromID" (ByVal hACam As Integer, ByVal Address As Integer, ByRef pError As Integer) As Integer

    End Class



    Public Class CArtCam

        Private m_hACam As Integer = 0
        Public m_Error As Integer = 1

        Public Function IsInit() As Boolean
            If 0 = m_hACam Then
                Return False
            Else
                Return True
            End If
        End Function


        Public Function GetDllVersion() As Integer
            Return CType(ArtCam.GetDllVersion(), Short)
        End Function

        Public Function GetDllType() As Integer
            Return ArtCam.GetDllVersion() / &H10000
        End Function

        Public Function GetLastError() As Integer
            Return ArtCam.GetLastError(m_hACam)
        End Function

        Public Function Initialize(ByVal hWnd As IntPtr) As Integer
            m_hACam = ArtCam.Initialize(hWnd)
            If 0 = m_hACam Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Function Release() As Integer
            Return ArtCam.Release(m_hACam)
        End Function

        Public Function Preview() As Integer
            Return ArtCam.Preview(m_hACam)
        End Function

        Public Function Record(ByVal lpAviName As String, ByVal RecTime As Integer, ByVal fShow As Integer) As Integer
            Return ArtCam.Record(m_hACam, lpAviName, RecTime, fShow)
        End Function

        Public Function CallBackPreview(ByVal hWnd As IntPtr, ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.CallBackPreviewA(m_hACam, hWnd, lpImage, Size, TopDown)
        End Function

        Public Function CallBackPreview(ByVal hWnd As IntPtr, ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.CallBackPreviewP(m_hACam, hWnd, lpImage, Size, TopDown)
        End Function

        Public Function SnapShot(ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.SnapShotA(m_hACam, lpImage, Size, TopDown)
        End Function

        Public Function SnapShot(ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.SnapShotP(m_hACam, lpImage, Size, TopDown)
        End Function

        Public Function Close() As Integer
            Return ArtCam.Close(m_hACam)
        End Function

        ' 1250
        Public Function Capture() As Integer
            Return ArtCam.Capture(m_hACam)
        End Function

        ' 1260
        Public Function Trigger(ByVal hWnd As IntPtr, ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.TriggerA(m_hACam, hWnd, lpImage, Size, TopDown)
        End Function

        Public Function Trigger(ByVal hWnd As IntPtr, ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.TriggerP(m_hACam, hWnd, lpImage, Size, TopDown)
        End Function

        Public Function SaveImage(ByVal lpSaveName As String, ByVal FileType As FILETYPE) As Integer
            Return ArtCam.SaveImage(m_hACam, lpSaveName, FileType)
        End Function

        Public Function SetPreviewWindow(ByVal hWnd As IntPtr, ByVal Left As Integer, ByVal Top As Integer, ByVal Right As Integer, ByVal Bottom As Integer) As Integer
            Return ArtCam.SetPreviewWindow(m_hACam, hWnd, Left, Top, Right, Bottom)
        End Function

        Public Function SetCaptureWindow(ByVal Width As Integer, ByVal Height As Integer, ByVal Frame As Integer) As Integer
            Return ArtCam.SetCaptureWindow(m_hACam, Width, Height, Frame)
        End Function

        Public Function SetCaptureWindowEx(ByVal HTotal As Integer, ByVal HStart As Integer, ByVal HEffective As Integer, ByVal VTotal As Integer, ByVal VStart As Integer, ByVal VEffective As Integer) As Integer
            Return ArtCam.SetCaptureWindowEx(m_hACam, HTotal, HStart, HEffective, VTotal, VStart, VEffective)
        End Function

        Public Function GetCaptureWindowEx(ByRef HTotal As Integer, ByRef HStart As Integer, ByRef HEffective As Integer, ByRef VTotal As Integer, ByRef VStart As Integer, ByRef VEffective As Integer) As Integer
            Return ArtCam.GetCaptureWindowEx(m_hACam, HTotal, HStart, HEffective, VTotal, VStart, VEffective)
        End Function

        Public Function SetColorMode(ByVal ColorMode As Integer) As Integer
            Return ArtCam.SetColorMode(m_hACam, ColorMode)
        End Function

        Public Function GetColorMode() As Integer
            Return ArtCam.GetColorMode(m_hACam)
        End Function

        Public Function SetCrossbar(ByVal Output As Integer, ByVal Input As Integer) As Integer
            Return ArtCam.SetCrossbar(m_hACam, Output, Input)
        End Function

        Public Function SetDeviceNumber(ByVal Number As Integer) As Integer
            Return ArtCam.SetDeviceNumber(m_hACam, Number)
        End Function

        Public Function GetDeviceName(ByVal index As Integer, ByVal szDeviceName As System.Text.StringBuilder, ByVal Size As Integer) As Integer
            Return ArtCam.GetDeviceName(m_hACam, index, szDeviceName, Size)
        End Function

        Public Function StartPreview() As Integer
            Return ArtCam.StartPreview(m_hACam)
        End Function

        Public Function StopPreview() As Integer
            Return ArtCam.StopPreview(m_hACam)
        End Function

        Public Function GetImage(ByVal lpImage As Byte(), ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.GetImageA(m_hACam, lpImage, Size, TopDown)
        End Function

        Public Function GetImage(ByVal lpImage As IntPtr, ByVal Size As Integer, ByVal TopDown As Integer) As Integer
            Return ArtCam.GetImageP(m_hACam, lpImage, Size, TopDown)
        End Function

        Public Function Width() As Integer
            Return ArtCam.Width(m_hACam)
        End Function

        Public Function Height() As Integer
            Return ArtCam.Height(m_hACam)
        End Function

        Public Function Fps() As Integer
            Return ArtCam.Fps(m_hACam)
        End Function

        Public Function SetCameraDlg(ByVal hWnd As IntPtr) As Integer
            Return ArtCam.SetCameraDlg(m_hACam, hWnd)
        End Function

        Public Function SetImageDlg(ByVal hWnd As IntPtr) As Integer
            Return ArtCam.SetImageDlg(m_hACam, hWnd)
        End Function

        Public Function SetAnalogDlg(ByVal hWnd As IntPtr) As Integer
            Return ArtCam.SetAnalogDlg(m_hACam, hWnd)
        End Function

        ' 1230
        Public Function SetIOPort(ByVal byteData As Integer, ByVal longData As Integer, ByVal Reserve As Integer) As Integer
            Return ArtCam.SetIOPort(m_hACam, byteData, longData, Reserve)
        End Function

        Public Function GetIOPort(ByVal byteData As IntPtr, ByVal longData As IntPtr, ByVal Reserve As Integer) As Integer
            Return ArtCam.GetIOPort(m_hACam, byteData, longData, Reserve)
        End Function

        Public Function SetFilterValue(ByVal FilterType As Integer, ByVal Value As Integer) As Integer
            Return ArtCam.SetFilterValue(m_hACam, FilterType, Value)
        End Function

        Public Function GetFilterValue(ByVal FilterType As Integer) As Integer
            Return ArtCam.GetFilterValue(m_hACam, FilterType, m_Error)
        End Function

        Public Function SetSubSample(ByVal SubSampleMode As Integer) As Integer
            Return ArtCam.SetSubSample(m_hACam, SubSampleMode)
        End Function

        Public Function GetSubSample() As Integer
            Return ArtCam.GetSubSample(m_hACam)
        End Function

        Public Function SetWaitTime(ByVal WaitTime As Integer) As Integer
            Return ArtCam.SetWaitTime(m_hACam, WaitTime)
        End Function

        Public Function GetWaitTime() As Integer
            Return ArtCam.GetWaitTime(m_hACam)
        End Function

        ' 1240
        Public Function SetMirrorV(ByVal Flg As Integer) As Integer
            Return ArtCam.SetMirrorV(m_hACam, Flg)
        End Function

        Public Function GetMirrorV() As Integer
            Return ArtCam.GetMirrorV(m_hACam)
        End Function

        Public Function SetMirrorH(ByVal Flg As Integer) As Integer
            Return ArtCam.SetMirrorH(m_hACam, Flg)
        End Function

        Public Function GetMirrorH() As Integer
            Return ArtCam.GetMirrorH(m_hACam)
        End Function

        Public Function SetBrightness(ByVal Value As Integer) As Integer
            Return ArtCam.SetBrightness(m_hACam, Value)
        End Function

        Public Function GetBrightness() As Integer
            Return ArtCam.GetBrightness(m_hACam, m_Error)
        End Function

        Public Function SetContrast(ByVal Value As Integer) As Integer
            Return ArtCam.SetContrast(m_hACam, Value)
        End Function

        Public Function GetContrast() As Integer
            Return ArtCam.GetContrast(m_hACam, m_Error)
        End Function

        Public Function SetHue(ByVal Value As Integer) As Integer
            Return ArtCam.SetHue(m_hACam, Value)
        End Function

        Public Function GetHue() As Integer
            Return ArtCam.GetHue(m_hACam, m_Error)
        End Function

        Public Function SetSaturation(ByVal Value As Integer) As Integer
            Return ArtCam.SetSaturation(m_hACam, Value)
        End Function

        Public Function GetSaturation() As Integer
            Return ArtCam.GetSaturation(m_hACam, m_Error)
        End Function

        Public Function SetSharpness(ByVal Value As Integer) As Integer
            Return ArtCam.SetSharpness(m_hACam, Value)
        End Function

        Public Function GetSharpness() As Integer
            Return ArtCam.GetSharpness(m_hACam, m_Error)
        End Function

        Public Function SetBayerGainRGB(ByVal Value As Integer) As Integer
            Return ArtCam.SetBayerGainRGB(m_hACam, Value)
        End Function

        Public Function GetBayerGainRGB() As Integer
            Return ArtCam.GetBayerGainRGB(m_hACam, m_Error)
        End Function

        Public Function SetBayerGainRed(ByVal Value As Integer) As Integer
            Return ArtCam.SetBayerGainRed(m_hACam, Value)
        End Function

        Public Function GetBayerGainRed() As Integer
            Return ArtCam.GetBayerGainRed(m_hACam, m_Error)
        End Function

        Public Function SetBayerGainGreen(ByVal Value As Integer) As Integer
            Return ArtCam.SetBayerGainGreen(m_hACam, Value)
        End Function

        Public Function GetBayerGainGreen() As Integer
            Return ArtCam.GetBayerGainGreen(m_hACam, m_Error)
        End Function

        Public Function SetBayerGainBlue(ByVal Value As Integer) As Integer
            Return ArtCam.SetBayerGainBlue(m_hACam, Value)
        End Function

        Public Function GetBayerGainBlue() As Integer
            Return ArtCam.GetBayerGainBlue(m_hACam, m_Error)
        End Function

        Public Function SetBayerGainAuto(ByVal Value As Integer) As Integer
            Return ArtCam.SetBayerGainAuto(m_hACam, Value)
        End Function

        Public Function GetBayerGainAuto() As Integer
            Return ArtCam.GetBayerGainAuto(m_hACam, m_Error)
        End Function

        Public Function SetGamma(ByVal Value As Integer) As Integer
            Return ArtCam.SetGamma(m_hACam, Value)
        End Function

        Public Function GetGamma() As Integer
            Return ArtCam.GetGamma(m_hACam, m_Error)
        End Function

        Public Function SetBayerMode(ByVal Value As Integer) As Integer
            Return ArtCam.SetBayerMode(m_hACam, Value)
        End Function

        Public Function GetBayerMode() As Integer
            Return ArtCam.GetBayerMode(m_hACam, m_Error)
        End Function

        Public Function SetGlobalGain(ByVal Value As Integer) As Integer
            Return ArtCam.SetGlobalGain(m_hACam, Value)
        End Function

        Public Function GetGlobalGain() As Integer
            Return ArtCam.GetGlobalGain(m_hACam, m_Error)
        End Function

        Public Function SetColorGainRed(ByVal Value As Integer) As Integer
            Return ArtCam.SetColorGainRed(m_hACam, Value)
        End Function

        Public Function GetColorGainRed() As Integer
            Return ArtCam.GetColorGainRed(m_hACam, m_Error)
        End Function

        Public Function SetColorGainGreen1(ByVal Value As Integer) As Integer
            Return ArtCam.SetColorGainGreen1(m_hACam, Value)
        End Function

        Public Function GetColorGainGreen1() As Integer
            Return ArtCam.GetColorGainGreen1(m_hACam, m_Error)
        End Function

        Public Function SetColorGainGreen2(ByVal Value As Integer) As Integer
            Return ArtCam.SetColorGainGreen2(m_hACam, Value)
        End Function

        Public Function GetColorGainGreen2() As Integer
            Return ArtCam.GetColorGainGreen2(m_hACam, m_Error)
        End Function

        Public Function SetColorGainBlue(ByVal Value As Integer) As Integer
            Return ArtCam.SetColorGainBlue(m_hACam, Value)
        End Function

        Public Function GetColorGainBlue() As Integer
            Return ArtCam.GetColorGainBlue(m_hACam, m_Error)
        End Function

        Public Function SetExposureTime(ByVal Value As Integer) As Integer
            Return ArtCam.SetExposureTime(m_hACam, Value)
        End Function

        Public Function GetExposureTime() As Integer
            Return ArtCam.GetExposureTime(m_hACam, m_Error)
        End Function

        ' 1260
        Public Function SetHalfClock(ByVal Value As Integer) As Integer
            Return ArtCam.SetHalfClock(m_hACam, Value)
        End Function

        Public Function GetHalfClock() As Integer
            Return ArtCam.GetHalfClock(m_hACam, m_Error)
        End Function

        Public Function SetAutoIris(ByVal Value As Integer) As Integer
            Return ArtCam.SetAutoIris(m_hACam, Value)
        End Function

        Public Function GetAutoIris() As Integer
            Return ArtCam.GetAutoIris(m_hACam, m_Error)
        End Function

        ' 1275
        Public Function SetSamplingRate(ByVal Value As Integer) As Integer
            Return ArtCam.SetSamplingRate(m_hACam, Value)
        End Function

        Public Function GetSamplingRate() As Integer
            Return ArtCam.GetSamplingRate(m_hACam, m_Error)
        End Function

        Public Function GetVideoFormat() As Integer
            Return ArtCam.GetVideoFormat(m_hACam, m_Error)
        End Function

        ' 1276
        Public Function WriteSromID(ByVal Address As Integer, ByVal Value As Integer) As Integer
            Return ArtCam.WriteSromID(m_hACam, Address, Value)
        End Function

        Public Function ReadSromID(ByVal Address As Integer) As Integer
            Return ArtCam.ReadSromID(m_hACam, Address, m_Error)
        End Function

    End Class

End Namespace
