//==========================================================
//	ARTRAY Camera / Capture Module Software Developer Kit
//						Preview/Callback function sample
//						VC++.NET Win32 Project
//
//									(C) 2002-2005 Artray
//==========================================================




//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Include file
//

#include <windows.h>
#include <stdio.h>

// Library include
#include "CArtCamSdk.hpp"
#include "CDib.h"


// Various menu ID 
enum {
	IDM_SAVE = WM_USER + 1,
	IDM_EXIT,

	IDM_PREVIEW,
	IDM_CALLBACK,
	IDM_SNAPSHOT,
	IDM_CAPTURE,
	IDM_TRIGGER,

	IDM_FORMAT,
	IDM_FILTER,
	IDM_ANALOG,

	IDM_CAM,
	IDM_USTC,
	IDM_CNV,
	IDM_130MI,
	IDM_200MI,
	IDM_300MI,
	IDM_150P,
	IDM_320P,
	IDM_200SH,
	IDM_098,
	IDM_036MI,
	IDM_500P,
	IDM_150P2,

	IDM_WIDEHI,
	IDM_WIDELOW,
	IDM_NORMALHI,
	IDM_NORMALLOW,
};



// WM_USER + 50 - WM_USER + 59 Reserved for device settings
#define IDM_DEVICE		WM_USER + 50


#define TIMER_ID	100
#define TIMER_TIME	50



CArtCamSdk	g_CArtCamSdk;
CDib		g_CDibCapture;		// Space for capture
CDib		g_CDibView;			// Space for display(only 24 bit color)

BOOL		g_Preview = -1;			// Preview display/Callback display
LONG		g_DllType = -1;			// DLL to be connected
LONG		g_DeviceCount = 0;		// Number of detected device


const int SUB_SAMPLE[] = { 1, 2, 4, 8 };
#define ArtCamGetWidth()	(g_CArtCamSdk.Width()  / SUB_SAMPLE[g_CArtCamSdk.GetSubSample()])
#define ArtCamGetHeight()	(g_CArtCamSdk.Height() / SUB_SAMPLE[g_CArtCamSdk.GetSubSample()])
#define ArtCamGetColor()	WORD(((g_CArtCamSdk.GetColorMode() + 7) / 8) * 8)


struct SCROLL_CTRL{
	POINT	Pos;

	BOOL	MoveFlg;
	POINT	Start;
};

SCROLL_CTRL	g_ScrollCtrl;

struct MENU_LIST{
	HMENU	File;
	HMENU	Preview;
	HMENU	Set;
	HMENU	Dll;
	HMENU	Device;
	HMENU	Rate;
};

MENU_LIST g_Menu;



TCHAR g_DllName[][256] = {
	"ArtCamSdk.dll",
	"",
	"ArtCamSdk_USTC.dll",
	"ArtCnvSdk.dll",
	"",
	"ArtCamSdk_130MI.dll",
	"ArtCamSdk_200MI.dll",
	"ArtCamSdk_300MI.dll",
	"ArtCamSdk_150P.dll",
	"ArtCamSdk_320P.dll",
	"ArtCamSdk_200SH.dll",
	"ArtCamSdk_098.dll",
	"ArtCamSdk_036MI.dll",
	"ArtCamSdk_500P.dll",
	"ArtCamSdk_150P2.dll"
};

const int DLLCOUNT = sizeof(g_DllName)/256;
LONG DefProc = NULL;


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Re-construct scroll bar
//

void ChangeScrollbar(HWND hWnd)
{
	if(0 == g_Preview){
		g_ScrollCtrl.Pos.x = 0;
		g_ScrollCtrl.Pos.y = 0;
		ShowScrollBar(hWnd, SB_HORZ, FALSE);
		ShowScrollBar(hWnd, SB_VERT, FALSE);
		return;
	}

	RECT rc;
	GetClientRect(hWnd, &rc);

	double dwScale = 1;

	LONG lWidth  = LONG(double(double(ArtCamGetWidth() ) * dwScale));
	LONG lHeight = LONG(double(double(ArtCamGetHeight()) * dwScale));

	LONG PosX = LONG(double(double(g_ScrollCtrl.Pos.x) * dwScale));
	LONG PosY = LONG(double(double(g_ScrollCtrl.Pos.y) * dwScale));


	SCROLLINFO sc;
	sc.fMask	= SIF_RANGE | SIF_POS | SIF_PAGE;

	LONG OffsetH = 16;
	LONG OffsetV = 16;


	if(rc.right + OffsetH >= lWidth){
		ShowScrollBar(hWnd, SB_HORZ, FALSE);
	} else {
		sc.nMin		= 0;
		sc.nMax		= lWidth;
		sc.nPage	= rc.right;
		sc.nPos		= PosX;
		SetScrollInfo(hWnd, SB_HORZ, &sc, TRUE);
	}

	if(rc.bottom + OffsetV >= lHeight){
		ShowScrollBar(hWnd, SB_VERT, FALSE);
	} else {
		sc.nMin		= 0;
		sc.nMax		= lHeight;
		sc.nPage	= rc.bottom;
		sc.nPos		= PosY;
		SetScrollInfo(hWnd, SB_VERT, &sc, TRUE);
	}


	double dwWidth  = double(ArtCamGetWidth ());
	double dwHeight = double(ArtCamGetHeight());

	if(dwWidth - (double(rc.right) / dwScale) < g_ScrollCtrl.Pos.x)
		g_ScrollCtrl.Pos.x = LONG(dwWidth - (double(rc.right) / dwScale));
	if(dwHeight - (double(rc.bottom) / dwScale) < g_ScrollCtrl.Pos.y)
		g_ScrollCtrl.Pos.y = LONG(dwHeight - (double(rc.bottom) / dwScale));
	if(0.0 > g_ScrollCtrl.Pos.x)
		g_ScrollCtrl.Pos.x = 0;
	if(0.0 > g_ScrollCtrl.Pos.y)
		g_ScrollCtrl.Pos.y = 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw image
//

LRESULT DrawImage(HWND hWnd, HDC hDC)
{
	BOOL Flg = FALSE;
	if(NULL == hDC){
		hDC = GetDC(hWnd);
		Flg = TRUE;
	}

	if(hDC){
		if(g_CDibView.GetImage()){
			// Convert to space for drawing
			g_CDibView.Convert24Bit(&g_CDibCapture);

			POINT pt;
			GetCursorPos(&pt);
			ScreenToClient(hWnd, &pt);

			pt.x += g_ScrollCtrl.Pos.x;
			pt.y += g_ScrollCtrl.Pos.y;

			// Draw the cross on cursor 
			g_CDibView.MoveTo(0, pt.y);
			g_CDibView.LineTo(g_CDibView.GetWidth(), pt.y);
			g_CDibView.MoveTo(pt.x, 0);
			g_CDibView.LineTo(pt.x, g_CDibView.GetHeight());

			// Display cursor coordinate
			TCHAR szText[256];
			wsprintf(szText, "(%d, %d)", pt.x, pt.y);
			g_CDibView.TextOut(pt.x+10, pt.y+10, szText);


			// Draw on picture
			::BitBlt(hDC, -g_ScrollCtrl.Pos.x, -g_ScrollCtrl.Pos.y, g_CDibView.GetWidth(), g_CDibView.GetHeight(),
							g_CDibView.GetDC(), 0, 0, SRCCOPY);
		}

		if(Flg){
			ReleaseDC(hWnd, hDC);
		}

		return 1;
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw line with SDK
//

BOOL OnPreview(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	g_CDibCapture.Release();
	g_CDibView.Release();


	// Set window to be displayed
	// When setting NULL to hWnd,it is possible to create new window and show it.
	RECT rc;
	GetClientRect(hWnd, &rc);
	g_CArtCamSdk.SetPreviewWindow(hWnd, 0, 0, rc.right, rc.bottom);


	// Display image
	g_CArtCamSdk.Preview();


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 0;

	ChangeScrollbar(hWnd);
	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Obtain image pointer and draw with application
//

BOOL OnCallBack(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	g_CDibCapture.Release();
	g_CDibView.Release();


	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	g_CArtCamSdk.SetPreviewWindow(hWnd, 0, 0, 0, 0);


	// Secure the space for taking in
	g_CDibCapture.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), ArtCamGetColor());

	// Secure the space for display
	g_CDibView.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), 24);
	g_CDibView.SelectPen(PS_SOLID, 1, RGB(255,0,0));
	g_CDibView.SelectPointFont(14, "MS UI Gothic");
	g_CDibView.SetTextColor(RGB(255,0,0));
	g_CDibView.SetBkMode(TRANSPARENT);

	g_CArtCamSdk.CallBackPreview(hWnd, g_CDibCapture.GetImage(), g_CDibCapture.GetSize(), 0);


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 1;

	ChangeScrollbar(hWnd);
	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Display one frame
//

BOOL OnSnapShot(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	g_CDibCapture.Release();
	g_CDibView.Release();


	// Secure the space for taking in
	g_CDibCapture.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), ArtCamGetColor());

	// Secure the space for display
	g_CDibView.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), 24);
	g_CDibView.SelectPen(PS_SOLID, 1, RGB(255,0,0));
	g_CDibView.SelectPointFont(14, "MS UI Gothic");
	g_CDibView.SetTextColor(RGB(255,0,0));
	g_CDibView.SetBkMode(TRANSPARENT);

	g_CArtCamSdk.SnapShot(g_CDibCapture.GetImage(), g_CDibCapture.GetSize(), 0);


	// Draw image
	DrawImage(hWnd, NULL);

	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = -1;

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Draw line with SDK
//

LRESULT OnCapture(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}


	// Release device
	g_CArtCamSdk.Close();


	// Set window to be displayed
	// When setting NULL to hWnd,it is possible to create new window and show it.
	RECT rc;
	GetClientRect(hWnd, &rc);
	g_CArtCamSdk.SetPreviewWindow(hWnd, 0, 0, rc.right, rc.bottom);


	// Display image
	// Display is controlled by SDK. Image information cannot be obtained.
	g_CArtCamSdk.Capture();


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_CHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_UNCHECKED);

	g_Preview = 3;


	// Release area for image
	g_CDibCapture.Release();
	g_CDibView.Release();


	// Secure the space for taking in
	g_CDibCapture.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), ArtCamGetColor());

	// Secure the space for display
	g_CDibView.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), 24);
	g_CDibView.SelectPen(PS_SOLID, 1, RGB(255,0,0));
	g_CDibView.SelectPointFont(14, "MS UI Gothic");
	g_CDibView.SetTextColor(RGB(255,0,0));
	g_CDibView.SetBkMode(TRANSPARENT);

	SetTimer(hWnd, TIMER_ID, TIMER_TIME, NULL);

	return TRUE;
}


LRESULT OnTimer(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(3 == g_Preview && TIMER_ID == wParam){
		KillTimer(hWnd, TIMER_ID);

		g_CArtCamSdk.SnapShot(g_CDibCapture.GetImage(), g_CDibCapture.GetSize(), 0);

		// Draw image
		DrawImage(hWnd, NULL);

		SetTimer(hWnd, TIMER_ID, TIMER_TIME, NULL);

		return 1;
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Trigger mode
//

BOOL OnTrigger(HWND hWnd)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}

	// Delete timer
	if(3 == g_Preview){
		KillTimer(hWnd, TIMER_ID);
	}


	// Release device
	g_CArtCamSdk.Close();


	// Release area for image
	g_CDibCapture.Release();
	g_CDibView.Release();


	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	g_CArtCamSdk.SetPreviewWindow(hWnd, 0, 0, 0, 0);


	// Secure the space for taking in
	g_CDibCapture.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), ArtCamGetColor());

	// Secure the space for display
	g_CDibView.Create(NULL, ArtCamGetWidth(), ArtCamGetHeight(), 24);
	g_CDibView.SelectPen(PS_SOLID, 1, RGB(255,0,0));
	g_CDibView.SelectPointFont(14, "MS UI Gothic");
	g_CDibView.SetTextColor(RGB(255,0,0));
	g_CDibView.SetBkMode(TRANSPARENT);

	g_CArtCamSdk.Trigger(hWnd, g_CDibCapture.GetImage(), g_CDibCapture.GetSize(), 0);


	// Check menu
	::CheckMenuItem(g_Menu.Preview, 0, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 1, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 3, MF_BYPOSITION | MF_UNCHECKED);
	::CheckMenuItem(g_Menu.Preview, 4, MF_BYPOSITION | MF_CHECKED);

	g_Preview = 4;

	ChangeScrollbar(hWnd);
	InvalidateRect(hWnd, NULL, TRUE);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Change DLL
//

BOOL OnDllChange(HWND hWnd, LONG DllType)
{
	// Release all devices before switching DLL
	// To display multiple images simulteously, you need to define CArtCamSdk plurally
	// When defined plurally, you do not need to relase to switch cameras
	g_CArtCamSdk.Release();
	g_CArtCamSdk.FreeLibrary();


	if(!g_CArtCamSdk.LoadLibrary(g_DllName[DllType])){
		TCHAR Error[256]="";
		wsprintf(Error, "%s is not found.\nDLL may have been relocated after executing", g_DllName[DllType]);
		MessageBox(hWnd, Error, "Error", MB_OK);
		return FALSE;
	}

	// Initialize is to be called first
	// By setting Window Handle here, WMLERROR can be obtained
	if(!g_CArtCamSdk.Initialize(hWnd)){
		MessageBox(hWnd, "Failed to initialize SDK", "Error", MB_OK);
		return FALSE;
	}


	// Check menu
	for(int i=0; i<DLLCOUNT; i++){
		::CheckMenuItem(g_Menu.Dll, i, MF_BYPOSITION | MF_UNCHECKED);
	}

	::CheckMenuItem(g_Menu.Dll, DllType, MF_BYPOSITION | MF_CHECKED);


	// Switch to ON or OFF of setting window
	switch(DllType){
	// CNV
	case 0:
	case 3:
		::EnableMenuItem(g_Menu.Set, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 2, MF_BYPOSITION | MF_GRAYED);

		::EnableMenuItem(g_Menu.Rate, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Rate, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Rate, 2, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Rate, 3, MF_BYPOSITION | MF_ENABLED);
	break;

	// Other
	default:
		::EnableMenuItem(g_Menu.Set, 0, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 1, MF_BYPOSITION | MF_ENABLED);
		::EnableMenuItem(g_Menu.Set, 2, MF_BYPOSITION | MF_ENABLED);

		::EnableMenuItem(g_Menu.Rate, 0, MF_BYPOSITION | MF_GRAYED);
		::EnableMenuItem(g_Menu.Rate, 1, MF_BYPOSITION | MF_GRAYED);
		::EnableMenuItem(g_Menu.Rate, 2, MF_BYPOSITION | MF_GRAYED);
		::EnableMenuItem(g_Menu.Rate, 3, MF_BYPOSITION | MF_GRAYED);
	break;
	}


	// Device modification menu
	for(int i=0; i<g_DeviceCount; i++){
		::DeleteMenu(g_Menu.Device, 0, MF_BYPOSITION | MF_STRING);
	}


	TCHAR szName[10][256];
	g_DeviceCount = g_CArtCamSdk.EnumDevice(szName);
	for(i=0; i<g_DeviceCount; i++){
		::InsertMenu(g_Menu.Device, i, MF_BYPOSITION | MF_STRING, IDM_DEVICE + i, szName[i]);
	}
	::CheckMenuItem(g_Menu.Device, 0, MF_BYPOSITION | MF_CHECKED);


//	g_CArtCamSdk.SetHalfClock(0);
//	g_CArtCamSdk.SetAutoIris(1);

	switch(g_Preview){
	case 0:	return OnPreview(hWnd);
	case 1:	return OnCallBack(hWnd);
	case 3:	return OnCapture(hWnd);
	case 4:	return OnTrigger(hWnd);
	}

	return FALSE;
}



//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Change device
//

BOOL OnDeviceChange(HWND hWnd, LONG Number)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}


	// Release the operative device before switching device
	g_CArtCamSdk.Close();

	// A device will be changed, if a camera is displayed after changing the number of a device now
	// Notes: A device is not changed in this function simple substance
	//   After calling this function, a device is changed by initializing a device
	g_CArtCamSdk.SetDeviceNumber(Number);

	for(int i=0; i<g_DeviceCount; i++){
		::CheckMenuItem(g_Menu.Device, i, MF_BYPOSITION | MF_UNCHECKED);
	}

	::CheckMenuItem(g_Menu.Device, Number, MF_BYPOSITION | MF_CHECKED);

	switch(g_Preview){
	case 0:	return OnPreview(hWnd);
	case 1:	return OnCallBack(hWnd);
	case 3:	return OnCapture(hWnd);
	case 4:	return OnTrigger(hWnd);
	}

	return FALSE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Switch to Sampling rate
//

BOOL OnRateChange(HWND hWnd, LONG Number)
{
	// Not selected DLL
	if(-1 == g_DllType){
		MessageBox(hWnd, "Select DLL for the camera connected", "Error", MB_OK);
		return FALSE;
	}


	// Release the operative device before switching device
	g_CArtCamSdk.Close();

	g_CArtCamSdk.SetSamplingRate(Number);

	for(int i=0; i<4; i++){
		::CheckMenuItem(g_Menu.Rate, i, MF_BYPOSITION | MF_UNCHECKED);
	}

	::CheckMenuItem(g_Menu.Rate, Number, MF_BYPOSITION | MF_CHECKED);

	switch(g_Preview){
	case 0:	return OnPreview(hWnd);
	case 1:	return OnCallBack(hWnd);
	case 3:	return OnCapture(hWnd);
	case 4:	return OnTrigger(hWnd);
	}

	return FALSE;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_CREATE
//

LRESULT OnCreate(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	TCHAR szFilePath[MAX_PATH]="";
	TCHAR szExePath[3][MAX_PATH]={"","",""};
	GetModuleFileName(::GetModuleHandle(NULL), szExePath[0], MAX_PATH);
	_splitpath(szExePath[0], szExePath[1], szExePath[2], NULL, NULL);
	wsprintf(szFilePath, "%s%sSample.ini", szExePath[1], szExePath[2]);

	g_DllType = ::GetPrivateProfileInt("Sample", "DllType", -1, szFilePath);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_COMMAND
//

LRESULT OnCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam)){
	// Save file
	case IDM_SAVE:
//		g_CArtCamSdk.SaveImage("Image.bmp", FILETYPE_BITMAP);
//		g_CArtCamSdk.SaveImage("Image.raw", FILETYPE_RAW);
		g_CArtCamSdk.SaveImage("Imageh.jpg", FILETYPE_JPEG_HIGH);
//		g_CArtCamSdk.SaveImage("Imagen.jpg", FILETYPE_JPEG_NOMAL);
//		g_CArtCamSdk.SaveImage("Imagel.jpg", FILETYPE_JPEG_LOW);
//		g_CArtCamSdk.SaveImage("Image.png", FILETYPE_PNG);
//		g_CArtCamSdk.SaveImage("Image.tif", FILETYPE_TIFF);
	return 1;

	// End
	case IDM_EXIT:
		::PostMessage(hWnd, WM_CLOSE, 0, 0);
	return 1;

	// Draw with SDK
	case IDM_PREVIEW:
		OnPreview(hWnd);
	return 1;

	// Obtain pointer of image and draw
	case IDM_CALLBACK:
		OnCallBack(hWnd);
	return 1;

	// Obtain A image
	case IDM_SNAPSHOT:
		OnSnapShot(hWnd);
	return 1;

	// Capture snapshot in a low
	case IDM_CAPTURE:
		OnCapture(hWnd);
	return 1;

	// Trigger mode
	case IDM_TRIGGER:
		OnTrigger(hWnd);
	return 1;

	// Camera settings
	case IDM_FORMAT:
		// When show camera setting dialog,recommend to release display once.
		// Stop image refresh with SDK even when not stop with application.
		// In case of stop with SDK,it restarts when close dialog
		// There is possibility of illegality processing 
		// if you don't correspond setting with application when you change setting and close dialog.
		if(3 == g_Preview)	KillTimer(hWnd, TIMER_ID);

		g_CArtCamSdk.Close();
		g_CArtCamSdk.SetCameraDlg(hWnd);

		switch(g_Preview){
		case 0:	return OnPreview(hWnd);
		case 1:	return OnCallBack(hWnd);
		case 3:	return OnCapture(hWnd);
		case 4:	return OnTrigger(hWnd);
		}
	return 1;

	// Filter settings
	case IDM_FILTER:
		g_CArtCamSdk.SetImageDlg(hWnd);
	return 1;

	// Analog settings
	case IDM_ANALOG:
		g_CArtCamSdk.SetAnalogDlg(hWnd);
	return 1;

	// Switch to DLL
	case IDM_CAM:		g_DllType = 0;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_USTC:		g_DllType = 2;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_CNV:		g_DllType = 3;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_130MI:		g_DllType = 5;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_200MI:		g_DllType = 6;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_300MI:		g_DllType = 7;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_150P:		g_DllType = 8;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_320P:		g_DllType = 9;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_200SH:		g_DllType = 10;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_098:		g_DllType = 11;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_036MI:		g_DllType = 12;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_500P:		g_DllType = 13;		OnDllChange(hWnd, g_DllType);	return 1;
	case IDM_150P2:		g_DllType = 14;		OnDllChange(hWnd, g_DllType);	return 1;

	// 
	case IDM_WIDEHI:		OnRateChange(hWnd, 0);	return 1;
	case IDM_WIDELOW:		OnRateChange(hWnd, 1);	return 1;
	case IDM_NORMALHI:		OnRateChange(hWnd, 2);	return 1;
	case IDM_NORMALLOW:		OnRateChange(hWnd, 3);	return 1;
	}


	// Change device
	for(int i=0; i<g_DeviceCount; i++){
		if(IDM_DEVICE + i == LOWORD(wParam)){
			OnDeviceChange(hWnd, i);
			return 1;
		}
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_PAINT
//

LRESULT OnPaint(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT psPaint;
	HDC hDC = BeginPaint(hWnd, &psPaint);
	if(hDC){
		DrawImage(hWnd, hDC);
		EndPaint(hWnd, &psPaint);
	}

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_ERROR
//

LRESULT OnError(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch(lParam){
	case ARTCAMSDK_NOERROR:		break;
	case ARTCAMSDK_NOT_INITIALIZE:	SetWindowText(hWnd, "Not initialized");	break;
	case ARTCAMSDK_DISABLEDDEVICE:	SetWindowText(hWnd, "Access to unavailable device was attempted");	break;
	case ARTCAMSDK_CREATETHREAD:	SetWindowText(hWnd, "Failure of thread creation for capturing");	break;
	case ARTCAMSDK_CREATEWINDOW:	SetWindowText(hWnd, "Window creation failed");	break;
	case ARTCAMSDK_OUTOFMEMORY:		SetWindowText(hWnd, "Not enough memory for image transfer/Failure for securing memory");	break;
	case ARTCAMSDK_CAMERASET:		SetWindowText(hWnd, "Error for camera (device) settings");	break;
	case ARTCAMSDK_CAPTURE:			SetWindowText(hWnd, "Capturing failed");	break;
	case ARTCAMSDK_PARAM:			SetWindowText(hWnd, "Wrong argument");	break;
	case ARTCAMSDK_DIRECTSHOW:		SetWindowText(hWnd, "Directshow initialization error");	break;
	case ARTCAMSDK_UNSUPPORTED:		SetWindowText(hWnd, "Not supported");	break;
	case ARTCAMSDK_UNKNOWN:			SetWindowText(hWnd, "Unknow error");	break;
	case ARTCAMSDK_CAPTURELOST:		SetWindowText(hWnd, "Device lost");	break;
	default:						SetWindowText(hWnd, "Unknow error");	break;
	}

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_GRAPHPAINT
//

LRESULT OnGraphPaint(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(NULL == wParam){
		return OnError(hWnd, wParam, lParam);
	}


	// Once this message is sent, latest image is stored at "g_pImage"
	LPGP_INFO pGpInfo = (LPGP_INFO)wParam;
	DrawImage(hWnd, NULL);


	TCHAR Fps[256]="";
	sprintf(Fps, "Sample  %.1f Frame", double(pGpInfo->lFps) / 10.0);
	SetWindowText(hWnd, Fps);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_DESTROY
//

LRESULT OnDestroy(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	// Call "Release" when you end use of camera. (also called "destructor").
	// You need to call "Initialise" when you re-use camera after "Release".
	g_CArtCamSdk.Release();
	g_CDibCapture.Release();
	g_CDibView.Release();


	TCHAR szFilePath[MAX_PATH]="";
	TCHAR szExePath[3][MAX_PATH]={"","",""};
	GetModuleFileName(::GetModuleHandle(NULL), szExePath[0], MAX_PATH);
	_splitpath(szExePath[0], szExePath[1], szExePath[2], NULL, NULL);
	wsprintf(szFilePath, "%s%sSample.ini", szExePath[1], szExePath[2]);


	TCHAR Data[32]="";
	wsprintf(Data, "%d", g_DllType);

	::WritePrivateProfileString("Sample", "DllType", Data, szFilePath);


	::PostQuitMessage(0);
	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WM_SIZE
//

LRESULT OnSize(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if(0 == g_Preview){
		HWND hChild = GetWindow(hWnd, GW_CHILD);

		RECT rc;
		GetClientRect(hWnd, &rc);


		// To avoiding changing aspect ratio, remove comment
/*
		// Calculation results in decimal figure. So apply floating-point calculations.
		double winW = rc.right;
		double winH = rc.bottom;
		double capW = ArtCamGetWidth();
		double capH = ArtCamGetHeight();

		if( (winW * capH) > (winH * capW) ){
			winW = winH * capW / capH;
			rc.right = LONG(winW);
		}
		else if( (winW * capH) < (winH * capW) ){
			winH = winW * capH / capW;
			rc.bottom = LONG(winH);
		}
*/
		SetWindowPos(hChild, HWND_TOP, 0, 0, rc.right, rc.bottom, SWP_NOMOVE | SWP_NOZORDER | SWP_SHOWWINDOW);
	}

	if(1 == g_Preview || 3 == g_Preview || 4 == g_Preview){
		ChangeScrollbar(hWnd);
	}

	return 1;
}


/////////////////////////////////////////////////////////////////////////////
//
//	WM_HSCROLL
//

LRESULT OnHScroll(HWND hWnd, UINT nSBCode, UINT nPos, HWND ScrollBar)
{
	SCROLLINFO sc;
	GetScrollInfo(hWnd, SB_HORZ, &sc);

	switch(nSBCode){
	// Scroll to th left end
	case SB_LEFT:
		SetScrollPos(hWnd, SB_HORZ, 0, TRUE);
	break;

	// Finish scrolling
	case SB_ENDSCROLL:
	break;

	// Scroll to left
	case SB_LINELEFT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) - sc.nPage/4, TRUE);
	break;

	// Scroll to right
	case SB_LINERIGHT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) + sc.nPage/4, TRUE);
	break;

	// Scroll to the left page
	case SB_PAGELEFT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) - sc.nPage, TRUE);
	break;

	// Scroll to the right page
	case SB_PAGERIGHT:
		SetScrollPos(hWnd, SB_HORZ, GetScrollPos(hWnd, SB_HORZ) + sc.nPage, TRUE);
	break;

	// Scroll to the right end
	case SB_RIGHT:
		SetScrollPos(hWnd, SB_HORZ, sc.nMax, TRUE);
	break;

	// Scroll to absolute position. The current position is appointed by argument "nPos".
	case SB_THUMBPOSITION:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_HORZ, nPos, TRUE);
	break;

	// Drag scroll box to the current position. The current position is appointed by argument "nPos". 
	case SB_THUMBTRACK:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_HORZ, nPos, TRUE);
	break;
	}

	g_ScrollCtrl.Pos.x = GetScrollPos(hWnd, SB_HORZ);
	DrawImage(hWnd, NULL);

	return 1;
}


/////////////////////////////////////////////////////////////////////////////
//
//	WM_VSCROLL
//

LRESULT OnVScroll(HWND hWnd, UINT nSBCode, UINT nPos, HWND ScrollBar)
{
	SCROLLINFO sc;
	GetScrollInfo(hWnd, SB_VERT, &sc);
	LONG pos = GetScrollPos(hWnd, SB_VERT);


	switch(nSBCode){
	// Scroll to th left end
	case SB_LEFT:
		SetScrollPos(hWnd, SB_VERT, 0, TRUE);
	break;

	// Finish scrolling
	case SB_ENDSCROLL:
	break;

	// Scroll to left
	case SB_LINELEFT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) - sc.nPage/4, TRUE);
	break;

	// Scroll to right
	case SB_LINERIGHT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) + sc.nPage/4, TRUE);
	break;

	// Scroll to the left page
	case SB_PAGELEFT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) - sc.nPage, TRUE);
	break;

	// Scroll to the right page
	case SB_PAGERIGHT:
		SetScrollPos(hWnd, SB_VERT, GetScrollPos(hWnd, SB_VERT) + sc.nPage, TRUE);
	break;

	// Scroll to the right end
	case SB_RIGHT:
		SetScrollPos(hWnd, SB_VERT, sc.nMax, TRUE);
	break;

	// Scroll to absolute position. The current position is appointed by argument "nPos".
	case SB_THUMBPOSITION:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_VERT, nPos, TRUE);
	break;

	// Drag scroll box to the current position. The current position is appointed by argument "nPos". 
	case SB_THUMBTRACK:
		nPos &= 0x0000FFFF;
		SetScrollPos(hWnd, SB_VERT, nPos, TRUE);
	break;
	}

	
	g_ScrollCtrl.Pos.y = GetScrollPos(hWnd, SB_VERT);
	DrawImage(hWnd, NULL);

	return 1;
}


/////////////////////////////////////////////////////////////////////////////
//
//	WM_MOUSEMOVE
//

LRESULT OnMouseMove(HWND hWnd, LONG xPos, LONG yPos)
{
	DrawImage(hWnd, NULL);

	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Window procedureWindow procedure
//

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg){
	case WM_CREATE:		return OnCreate			(hWnd, wParam, lParam);
	case WM_COMMAND:	return OnCommand		(hWnd, wParam, lParam);
	case WM_PAINT:		return OnPaint			(hWnd, wParam, lParam);
	case WM_GRAPHPAINT:	return OnGraphPaint		(hWnd, wParam, lParam);
	case WM_ERROR:		return OnError			(hWnd, wParam, lParam);
	case WM_DESTROY:	return OnDestroy		(hWnd, wParam, lParam);
	case WM_SIZE:		return OnSize			(hWnd, wParam, lParam);
	case WM_TIMER:		return OnTimer			(hWnd, wParam, lParam);
	case WM_HSCROLL:	return OnHScroll		(hWnd, LOWORD(wParam), HIWORD(wParam), (HWND)lParam);
	case WM_VSCROLL:	return OnVScroll		(hWnd, LOWORD(wParam), HIWORD(wParam), (HWND)lParam);
	case WM_MOUSEMOVE:	return OnMouseMove		(hWnd, LOWORD(lParam), HIWORD(lParam));
	default:			return ::DefWindowProc	(hWnd, uMsg, wParam, lParam);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	Window initializationInitialize window
//

HWND InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	// Registrate window classRegister window class
	WNDCLASSEX wndclassex;
	ZeroMemory(&wndclassex,		sizeof(wndclassex));
	wndclassex.cbSize			= sizeof(wndclassex);
	wndclassex.style			= 0;
	wndclassex.lpfnWndProc		= WndProc;
	wndclassex.cbClsExtra		= 0;
	wndclassex.cbWndExtra		= 0;
	wndclassex.hInstance		= hInstance;
	wndclassex.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	wndclassex.hCursor			= LoadCursor(NULL, IDC_ARROW);
	wndclassex.hbrBackground	= reinterpret_cast<HBRUSH>(::GetStockObject(BLACK_BRUSH));
	wndclassex.lpszClassName	= "Sample";
	wndclassex.lpszMenuName		= 0;
	wndclassex.hIconSm			= NULL;

	if(0 == ::RegisterClassEx(&wndclassex))
		return NULL;

	// Window size settingWindow size settings
	RECT rcWndSize = {0, 0, 900, 600};
	DWORD dwStyle = WS_POPUPWINDOW | WS_CAPTION | WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL;
	if(0 == ::AdjustWindowRectEx(&rcWndSize, dwStyle, TRUE, NULL))
		return NULL;

	// Main window createMain window generation
	HWND hWnd = ::CreateWindowEx(NULL, "Sample", "Sample", dwStyle,
			(::GetSystemMetrics(SM_CXSCREEN) - (rcWndSize.right - rcWndSize.left)) / 2,
			(::GetSystemMetrics(SM_CYSCREEN) - (rcWndSize.bottom - rcWndSize.top)) / 2,
			rcWndSize.right - rcWndSize.left, rcWndSize.bottom - rcWndSize.top,
			HWND_DESKTOP, NULL, hInstance, NULL);

	if(NULL == hWnd)
		return NULL;


	// Create menu

	// File menu
	g_Menu.File = ::CreateMenu();
	::InsertMenu(g_Menu.File, 0, MF_BYPOSITION | MF_STRING, IDM_SAVE, "Save(&S)");
	::InsertMenu(g_Menu.File, 1, MF_BYPOSITION | MF_STRING, IDM_EXIT, "End(&X)");

	// Display type menu
	g_Menu.Preview = ::CreateMenu();
	::InsertMenu(g_Menu.Preview, 0, MF_BYPOSITION | MF_STRING, IDM_PREVIEW, "Preview(&P)");
	::InsertMenu(g_Menu.Preview, 1, MF_BYPOSITION | MF_STRING, IDM_CALLBACK, "Callback(&B)");
	::InsertMenu(g_Menu.Preview, 2, MF_BYPOSITION | MF_STRING, IDM_SNAPSHOT, "Snapshot(&S)");
	::InsertMenu(g_Menu.Preview, 3, MF_BYPOSITION | MF_STRING, IDM_CAPTURE, "Capture(&C)");
	::InsertMenu(g_Menu.Preview, 4, MF_BYPOSITION | MF_STRING, IDM_TRIGGER, "Trigger(&T)");

	// Settings menu
	g_Menu.Set = ::CreateMenu();
	::InsertMenu(g_Menu.Set, 0, MF_BYPOSITION | MF_STRING, IDM_FORMAT, "Format settings(&C)");
	::InsertMenu(g_Menu.Set, 1, MF_BYPOSITION | MF_STRING, IDM_FILTER, "Filter settings(&F)");
	::InsertMenu(g_Menu.Set, 2, MF_BYPOSITION | MF_STRING, IDM_ANALOG, "Analog settings(&A)");

	// DLL selective menu
	g_Menu.Dll = ::CreateMenu();
	::InsertMenu(g_Menu.Dll,  0, MF_BYPOSITION | MF_STRING, IDM_CAM,   "DirectShow Camera");
	::InsertMenu(g_Menu.Dll,  1, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
	::InsertMenu(g_Menu.Dll,  2, MF_BYPOSITION | MF_STRING, IDM_USTC,  "ARTUST-0209C");
	::InsertMenu(g_Menu.Dll,  3, MF_BYPOSITION | MF_STRING, IDM_CNV,   "ARTCNV");
	::InsertMenu(g_Menu.Dll,  4, MF_BYPOSITION | MF_SEPARATOR, NULL, NULL );
	::InsertMenu(g_Menu.Dll,  5, MF_BYPOSITION | MF_STRING, IDM_130MI, "ARTCAM-130MI");
	::InsertMenu(g_Menu.Dll,  6, MF_BYPOSITION | MF_STRING, IDM_200MI, "ARTCAM-200MI");
	::InsertMenu(g_Menu.Dll,  7, MF_BYPOSITION | MF_STRING, IDM_300MI, "ARTCAM-300MI");
	::InsertMenu(g_Menu.Dll,  8, MF_BYPOSITION | MF_STRING, IDM_150P,  "ARTCAM-150P");
	::InsertMenu(g_Menu.Dll,  9, MF_BYPOSITION | MF_STRING, IDM_320P,  "ARTCAM-320P");
	::InsertMenu(g_Menu.Dll, 10, MF_BYPOSITION | MF_STRING, IDM_200SH, "ARTCAM-200SH");
	::InsertMenu(g_Menu.Dll, 11, MF_BYPOSITION | MF_STRING, IDM_098,   "ARTCAM-098");
	::InsertMenu(g_Menu.Dll, 12, MF_BYPOSITION | MF_STRING, IDM_036MI, "ARTCAM-036MI");
	::InsertMenu(g_Menu.Dll, 13, MF_BYPOSITION | MF_STRING, IDM_500P,  "ARTCAM-500P");
	::InsertMenu(g_Menu.Dll, 14, MF_BYPOSITION | MF_STRING, IDM_500P,  "ARTCAM-150P2");

	// Sampling rate menu
	g_Menu.Rate = ::CreateMenu();
	::InsertMenu(g_Menu.Rate,  0, MF_BYPOSITION | MF_STRING, IDM_WIDEHI, "Wide HiSpeed");
	::InsertMenu(g_Menu.Rate,  1, MF_BYPOSITION | MF_STRING, IDM_WIDELOW, "Wide LowSpeed");
	::InsertMenu(g_Menu.Rate,  2, MF_BYPOSITION | MF_STRING, IDM_NORMALHI, "Normal HiSpeed");
	::InsertMenu(g_Menu.Rate,  3, MF_BYPOSITION | MF_STRING, IDM_NORMALLOW, "Normal LowSpeed");

	// Check if there is DLL
	for(int i=0; i<DLLCOUNT; i++){
		CArtCamSdk ArtCam;
		if(!ArtCam.LoadLibrary(g_DllName[i])){
			::EnableMenuItem(g_Menu.Dll, i, MF_BYPOSITION | MF_GRAYED);
		}
	}



	// Device modification menu
	g_Menu.Device = ::CreateMenu();

	// Main menu
	HMENU hMenu = ::CreateMenu();
	::InsertMenu(hMenu, 0, MF_POPUP | MF_STRING, (UINT)g_Menu.File,    "File(&F)");
	::InsertMenu(hMenu, 1, MF_POPUP | MF_STRING, (UINT)g_Menu.Preview, "Display(&V)");
	::InsertMenu(hMenu, 2, MF_POPUP | MF_STRING, (UINT)g_Menu.Set,     "Settings(&S)");
	::InsertMenu(hMenu, 3, MF_POPUP | MF_STRING, (UINT)g_Menu.Dll,     "DLL(&L)");
	::InsertMenu(hMenu, 4, MF_POPUP | MF_STRING, (UINT)g_Menu.Device,  "Device(&D)");
	::InsertMenu(hMenu, 5, MF_POPUP | MF_STRING, (UINT)g_Menu.Rate,    "Rate(&D)");


	::SetMenu(hWnd, hMenu);


	// Focus settings
	::SetFocus(hWnd);
	::ShowWindow(hWnd, nCmdShow);
	if(0 == ::UpdateWindow(hWnd))
		return NULL;

	// Initialize camera
	if(-1 != g_DllType)
		OnDllChange(hWnd, g_DllType);

	SetTimer(hWnd, 1, 100, NULL);
	return hWnd;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//
//	WinMain
//

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmLine, int nCmdShow)
{
	HWND hWnd = InitWindow(hInstance, nCmdShow);

	// Message roop
	MSG msg;
	ZeroMemory(&msg, sizeof(msg));
	while(::GetMessage(&msg, NULL, 0, 0)){
		::TranslateMessage(&msg);
		::DispatchMessage(&msg);
	}

	return msg.wParam;
}



