//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;


TCHAR DllName[][256] = {
	"ArtCamSdk.dll",
	"",
	"ArtCamSdk_USTC.dll",
	"ArtCnvSdk.dll",
	"",
	"ArtCamSdk_130MI.dll",
	"ArtCamSdk_200MI.dll",
	"ArtCamSdk_300MI.dll",
	"ArtCamSdk_150P.dll",
	"ArtCamSdk_320P.dll",
	"ArtCamSdk_200SH.dll",
	"ArtCamSdk_098.dll",
	"ArtCamSdk_036MI.dll",
	"ArtCamSdk_500P.dll",
	"ArtCamSdk_150P2.dll",
	"ArtCamSdk_500MI.dll",
	"ArtCamSdk_T111.dll",
	"ArtCamSdk_T112.dll",
	"ArtCamSdk_150P3.dll",
	"ArtCamSdk_130MIV.dll",
	"ArtCamSdk_150P3V.dll",
	"ArtCamSdk_267KY.dll",
	"ArtCamSdk_274KY.dll",
	"",
	"ArtCamSdk_Sata.dll"
};

const int DLLCOUNT = sizeof(DllName)/256;


//---------------------------------------------------------------------------
LRESULT CALLBACK MsgProc(int nCode, WPARAM wp, LPARAM lp)
{
	if(nCode < 0)
		return CallNextHookEx(Form1->m_Hook, nCode, wp, lp);

	if(HC_ACTION == nCode){
		PMsg Msg = (PMsg)lp;

		// Display code. Sent from SDK when the new image is captured
		if(WM_GRAPHPAINT == Msg->message){
			Form1->DrawImage();
		}

		// Error Code
		if(WM_ERROR == Msg->message){
			switch(Msg->lParam){
			case ARTCAMSDK_NOERROR:         Form1->Caption = "Sample";	break;
			case ARTCAMSDK_NOT_INITIALIZE:  Form1->Caption = "Not initialized";	break;
			case ARTCAMSDK_DISABLEDDEVICE:  Form1->Caption = "Access to unavailable device was attempted";	break;
			case ARTCAMSDK_CREATETHREAD:    Form1->Caption = "Failure of thread creation for capturing";	break;
			case ARTCAMSDK_CREATEWINDOW:    Form1->Caption = "Window creation failed";	break;
			case ARTCAMSDK_OUTOFMEMORY:     Form1->Caption = "Not enough memory for image transfer/Failure for securing memory";	break;
			case ARTCAMSDK_CAMERASET:       Form1->Caption = "Error for camera (device) settings";	break;
			case ARTCAMSDK_CAPTURE:         Form1->Caption = "Capturing failed";	break;
			case ARTCAMSDK_PARAM:           Form1->Caption = "Wrong argument";	break;
			case ARTCAMSDK_DIRECTSHOW:      Form1->Caption = "Directshow initialization error";	break;
			case ARTCAMSDK_UNSUPPORTED:     Form1->Caption = "Not supported";	break;
			case ARTCAMSDK_UNKNOWN:         Form1->Caption = "Unknow error";	break;
			case ARTCAMSDK_CAPTURELOST:     Form1->Caption = "Device lost";	break;
			case ARTCAMSDK_FILENOTFOUND:    Form1->Caption = "File not found";	break;
			case ARTCAMSDK_FPGASET:         Form1->Caption = "FPGA setting error";	break;
			default:                        Form1->Caption = "Unknow error";	break;
			}
		}
	}

	return CallNextHookEx(Form1->m_Hook, nCode, wp, lp);
}


//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
		: TForm(Owner)
		, m_DllType(-1)
		, m_PreviewMode(1)
		, m_Hook(NULL)
{
    m_BmpDraw[0] = NULL;
    m_BmpDraw[1] = NULL;
	Timer1->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::FormCreate(TObject *Sender)
{
	for(int i=0; i<15; i++){
		if(!m_CArtCamSdk[0].LoadLibrary(DllName[i])){
			(*PopupDll)[i]->Enabled = false;
		}
		m_CArtCamSdk[0].FreeLibrary();
	}

	m_Hook = SetWindowsHookEx(WH_GETMESSAGE, (HOOKPROC)MsgProc, 0, GetCurrentThreadId());
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SaveClick(TObject *Sender)
{
	m_CArtCamSdk[0].SaveImage("Temp1.jpg", FILETYPE_JPEG_NOMAL);
	m_CArtCamSdk[1].SaveImage("Temp2.jpg", FILETYPE_JPEG_NOMAL);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ExitClick(TObject *Sender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::PreviewClick(TObject *Sender)
{
	// Not selected DLL
	if(-1 == m_DllType){
		ShowMessage("Select DLL for the camera connected");
		return;
	}

	// Delete timer
	if(3 == m_PreviewMode){
		Timer1->Enabled = false;
	}

	// Release device
	m_CArtCamSdk[0].Close();
	m_CArtCamSdk[1].Close();

	// Set window to be displayed
	// New window is created when NULL is set in Handle.
	TRect rc = GetClientRect();
	m_CArtCamSdk[0].SetPreviewWindow(Handle, 0, 0, rc.right/2, rc.bottom);
	m_CArtCamSdk[1].SetPreviewWindow(Handle, rc.right/2, 0, rc.right, rc.bottom);

	// Display image
	// Display is controlled by SDK. Image information cannot be obtained.
	m_CArtCamSdk[0].Preview();
	m_CArtCamSdk[1].Preview();

	// Check menu
	ShowPreview->Checked  = true;
	ShowCallBack->Checked = false;
	ShowCapture->Checked  = false;
	ShowTrigger->Checked  = false;

	Invalidate();
	m_PreviewMode = 0;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::CallBackClick(TObject *Sender)
{
	// Not selected DLL
	if(-1 == m_DllType){
		ShowMessage("Select DLL for the camera connected");
		return;
	}

	// Delete timer
	if(3 == m_PreviewMode){
		Timer1->Enabled = false;
	}

	// Release device
	m_CArtCamSdk[0].Close();
	m_CArtCamSdk[1].Close();

	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	m_CArtCamSdk[0].SetPreviewWindow(Handle, 0, 0, 0, 0);
	m_CArtCamSdk[1].SetPreviewWindow(Handle, 0, 0, 0, 0);

	// Creat area for capturing
	CreateBitmap();

	// Capture image
	m_CArtCamSdk[0].CallBackPreview(Handle, (LPBYTE)m_BmpDraw[0]->ScanLine[m_BmpDraw[0]->Height-1], ArtCamGetSize(0), 0);
	m_CArtCamSdk[1].CallBackPreview(Handle, (LPBYTE)m_BmpDraw[1]->ScanLine[m_BmpDraw[1]->Height-1], ArtCamGetSize(1), 0);

	// Check menu
	ShowPreview->Checked  = false;
	ShowCallBack->Checked = true;
	ShowCapture->Checked  = false;
	ShowTrigger->Checked  = false;

	Invalidate();
	m_PreviewMode = 1;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SnapShotClick(TObject *Sender)
{
	// Not selected DLL
	if(-1 == m_DllType){
		ShowMessage("Select DLL for the camera connected");
		return;
	}

	// Delete timer
	if(3 == m_PreviewMode){
		Timer1->Enabled = false;
	}

	// Release device
	m_CArtCamSdk[0].Close();
	m_CArtCamSdk[1].Close();

	// Creat area for capturing
	CreateBitmap();

	// Capture image
	m_CArtCamSdk[0].SnapShot((LPBYTE)m_BmpDraw[0]->ScanLine[m_BmpDraw[0]->Height-1], ArtCamGetSize(0), 0);
	m_CArtCamSdk[1].SnapShot((LPBYTE)m_BmpDraw[1]->ScanLine[m_BmpDraw[1]->Height-1], ArtCamGetSize(1), 0);

	// Draw image
	DrawImage();

	// Check menu
	ShowPreview->Checked  = false;
	ShowCallBack->Checked = false;
	ShowCapture->Checked  = false;
	ShowTrigger->Checked  = false;

//	Invalidate();
	m_PreviewMode = -1;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::CaptureClick(TObject *Sender)
{
	// Not selected DLL
	if(-1 == m_DllType){
		ShowMessage("Select DLL for the camera connected");
		return;
	}

	// Release device
	m_CArtCamSdk[0].Close();
	m_CArtCamSdk[1].Close();

	// Set window to be displayed
	// New window is created when NULL is set in Handle.
	m_CArtCamSdk[0].SetPreviewWindow(Handle, 0, 0, 0, 0);
	m_CArtCamSdk[1].SetPreviewWindow(Handle, 0, 0, 0, 0);

	// Creat area for capturing
	CreateBitmap();

	// Display image
	m_CArtCamSdk[0].Capture();
	m_CArtCamSdk[1].Capture();

	// Check menu
	ShowPreview->Checked  = false;
	ShowCallBack->Checked = false;
	ShowCapture->Checked  = true;
	ShowTrigger->Checked  = false;

	Invalidate();
	m_PreviewMode = 3;

	Timer1->Enabled = true;
	Timer1->Interval = 200;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::TriggerClick(TObject *Sender)
{
	// Not selected DLL
	if(-1 == m_DllType){
		ShowMessage("Select DLL for the camera connected");
		return;
	}

	// Delete timer
	if(3 == m_PreviewMode){
		Timer1->Enabled = false;
	}

	// Release device
	m_CArtCamSdk[0].Close();
	m_CArtCamSdk[1].Close();

	// If drawing by yourself, set all window size to 0.
	// An automatic display can be performed, if window size is set up even when using CallBackPreview
	m_CArtCamSdk[0].SetPreviewWindow(Handle, 0, 0, 0, 0);
	m_CArtCamSdk[1].SetPreviewWindow(Handle, 0, 0, 0, 0);

	// Creat area for capturing
	CreateBitmap();

	// Capture image
	m_CArtCamSdk[0].Trigger(Handle, (LPBYTE)m_BmpDraw[0]->ScanLine[m_BmpDraw[0]->Height-1], ArtCamGetSize(0), 0);
	m_CArtCamSdk[1].Trigger(Handle, (LPBYTE)m_BmpDraw[1]->ScanLine[m_BmpDraw[1]->Height-1], ArtCamGetSize(1), 0);

	// Check menu
	ShowPreview->Checked  = false;
	ShowCallBack->Checked = false;
	ShowCapture->Checked  = false;
	ShowTrigger->Checked  = true;

	Invalidate();
	m_PreviewMode = 4;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SetCameraClick1(TObject *Sender)
{
	m_CArtCamSdk[0].SetCameraDlg(Handle);

	Invalidate();
	switch(m_PreviewMode){
	case 0:	PreviewClick(Sender); break;
	case 1:	CallBackClick(Sender); break;
	case 3:	CaptureClick(Sender); break;
	case 4:	TriggerClick(Sender); break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SetCameraClick2(TObject *Sender)
{
	m_CArtCamSdk[1].SetCameraDlg(Handle);

	Invalidate();
	switch(m_PreviewMode){
	case 0:	PreviewClick(Sender); break;
	case 1:	CallBackClick(Sender); break;
	case 3:	CaptureClick(Sender); break;
	case 4:	TriggerClick(Sender); break;
	}
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SetFilterClick1(TObject *Sender)
{
	m_CArtCamSdk[0].SetImageDlg(Handle);
}//---------------------------------------------------------------------------
void __fastcall TForm1::SetFilterClick2(TObject *Sender)
{
	m_CArtCamSdk[1].SetImageDlg(Handle);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::SetAnalogClick1(TObject *Sender)
{
	m_CArtCamSdk[0].SetAnalogDlg(Handle);
}//---------------------------------------------------------------------------
void __fastcall TForm1::SetAnalogClick2(TObject *Sender)
{
	m_CArtCamSdk[1].SetAnalogDlg(Handle);
}
//---------------------------------------------------------------------------
void TForm1::OnDllChange(TObject *Sender, LONG Number, LONG CameraType)
{
	// Release all devices before switching DLL
	// To display multiple images simulteously, you need to define CArtCamSdk plurally
	// When defined plurally, you do not need to relase to switch cameras
	m_CArtCamSdk[0].Release();
	m_CArtCamSdk[1].Release();
	m_CArtCamSdk[0].FreeLibrary();
	m_CArtCamSdk[1].FreeLibrary();

	m_DllType = Number;

	if(!m_CArtCamSdk[0].LoadLibrary(DllName[m_DllType])){
		TCHAR Error[256]="";
		wsprintf(Error, "%s is not found. \nDLL may have been relocated after executing", DllName[m_DllType]);
		ShowMessage(Error);
		return;
	}
	if(!m_CArtCamSdk[1].LoadLibrary(DllName[m_DllType])){
		TCHAR Error[256]="";
		wsprintf(Error, "%s is not found. \nDLL may have been relocated after executing", DllName[m_DllType]);
		ShowMessage(Error);
		return;
	}

	// Check selected DLL
	for(int i=0; i<15; i++){
		(*PopupDll)[i]->Checked = false;
	}
	(*PopupDll)[m_DllType]->Checked = true;


	// Initialize is to be called firstCall "Initialize" first
	// By setting Window Handle here, WMLERROR can be obtainedBy setting "Window handle" here, "WM_ERROR" can be obtained
	if(!m_CArtCamSdk[0].Initialize(Handle)){
		ShowMessage("Failed to initialize SDK");
		return;
	}
	if(!m_CArtCamSdk[1].Initialize(Handle)){
		ShowMessage("Failed to initialize SDK");
		return;
	}

	//Sata cameras are not supported for multiple capturing
/*
	if(-1 != CameraType && ARTCAM_CAMERATYPE_SATA == HIWORD(m_CArtCamSdk[0].GetDllVersion())){
		m_CArtCamSdk[0].SetCameraType(CameraType);
	}
	if(-1 != CameraType && ARTCAM_CAMERATYPE_SATA == HIWORD(m_CArtCamSdk[1].GetDllVersion())){
		m_CArtCamSdk[1].SetCameraType(CameraType);
	}
*/

	m_CArtCamSdk[0].SetDeviceNumber(0);
	m_CArtCamSdk[1].SetDeviceNumber(1);

	// To operate 2 cameras simultenously, set camera clock to half.
	m_CArtCamSdk[0].SetHalfClock(TRUE);
	m_CArtCamSdk[1].SetHalfClock(TRUE);


	switch(m_PreviewMode){
	case 0:	PreviewClick(Sender); break;
	case 1:	CallBackClick(Sender); break;
	case 3:	CaptureClick(Sender); break;
	case 4:	TriggerClick(Sender); break;
	}
}
//---------------------------------------------------------------------------
void TForm1::DrawImage(void)
{
	// Draw image
	Image1->Canvas->Draw(0, 0, m_BmpDraw[0]);
 	Image2->Canvas->Draw(0, 0, m_BmpDraw[1]);
}
//---------------------------------------------------------------------------
void TForm1::CreateBitmap(void)
{
	// Release area for image
	if(m_BmpDraw[0])	delete m_BmpDraw[0];
	if(m_BmpDraw[1])	delete m_BmpDraw[1];
	m_BmpDraw[0] = NULL;
	m_BmpDraw[1] = NULL;


	// Creat area for display
	for(int i=0; i<2; i++){
		m_BmpDraw[i] = new Graphics::TBitmap;
		m_BmpDraw[i]->Width  = ArtCamGetWidth (i);
		m_BmpDraw[i]->Height = ArtCamGetHeight(i);
		m_BmpDraw[i]->Canvas->Pen->Color = clRed;

		// Generate palette when 8 bit monochrome image
		if(1 == ArtCamGetBpp(i)){
			m_BmpDraw[i]->PixelFormat = pf8bit;
			TMaxLogPalette pal;
			for(int i=0; i<256; i++){
				pal.palPalEntry[i].peRed   = i;
				pal.palPalEntry[i].peGreen = i;
				pal.palPalEntry[i].peBlue  = i;
				pal.palPalEntry[i].peFlags = 0;
			}
			pal.palNumEntries = 256;
			pal.palVersion = 0x0300;
			m_BmpDraw[i]->Palette = CreatePalette(PLogPalette(&pal));
		}
		// 24 bit color
		else if(3 == ArtCamGetBpp(i)){
			m_BmpDraw[i]->PixelFormat = pf24bit;
		}
	}

	// To use other color number,
	// Sample_BCB6_Graphic&Allbpp
	// please refer to **

}
//---------------------------------------------------------------------------
void __fastcall TForm1::DSCameraClick    (TObject *Sender) { OnDllChange(Sender,  0, -1); }
void __fastcall TForm1::ARTUSTClick      (TObject *Sender) { OnDllChange(Sender,  2, -1); }
void __fastcall TForm1::ARTCNVClick      (TObject *Sender) { OnDllChange(Sender,  3, -1); }
void __fastcall TForm1::ARTCAM130MIClick (TObject *Sender) { OnDllChange(Sender,  5, -1); }
void __fastcall TForm1::ARTCAM200MIClick (TObject *Sender) { OnDllChange(Sender,  6, -1); }
void __fastcall TForm1::ARTCAM300MIClick (TObject *Sender) { OnDllChange(Sender,  7, -1); }
void __fastcall TForm1::ARTCAM150PClick  (TObject *Sender) { OnDllChange(Sender,  8, -1); }
void __fastcall TForm1::ARTCAM320PClick  (TObject *Sender) { OnDllChange(Sender,  9, -1); }
void __fastcall TForm1::ARTCAM200SHClick (TObject *Sender) { OnDllChange(Sender, 10, -1); }
void __fastcall TForm1::ARTCAM098Click   (TObject *Sender) { OnDllChange(Sender, 11, -1); }
void __fastcall TForm1::ARTCAM036MIClick (TObject *Sender) { OnDllChange(Sender, 12, -1); }
void __fastcall TForm1::ARTCAM500PClick  (TObject *Sender) { OnDllChange(Sender, 13, -1); }
void __fastcall TForm1::ARTCAM150P2Click (TObject *Sender) { OnDllChange(Sender, 14, -1); }
void __fastcall TForm1::ARTCAM500MIClick (TObject *Sender) { OnDllChange(Sender, 15, -1); }
void __fastcall TForm1::ARTCAMT111Click  (TObject *Sender) { OnDllChange(Sender, 16, -1); }
void __fastcall TForm1::ARTCAMT112Click  (TObject *Sender) { OnDllChange(Sender, 17, -1); }
void __fastcall TForm1::ARTCAM150P3Click (TObject *Sender) { OnDllChange(Sender, 18, -1); }
void __fastcall TForm1::ARTCAM130MIVClick(TObject *Sender) { OnDllChange(Sender, 19, -1); }
void __fastcall TForm1::ARTCAM150P3VClick(TObject *Sender) { OnDllChange(Sender, 20, -1); }
void __fastcall TForm1::ARTCAM267KYClick (TObject *Sender) { OnDllChange(Sender, 21, -1); }
void __fastcall TForm1::ARTCAM274KYClick (TObject *Sender) { OnDllChange(Sender, 22, -1); }

void __fastcall TForm1::ARTCAMSLVDSClick (TObject *Sender) { OnDllChange(Sender, 24, 0); }
void __fastcall TForm1::ARTCAMS300MIClick(TObject *Sender) { OnDllChange(Sender, 24, 1); }
void __fastcall TForm1::ARTCAMS500MIClick(TObject *Sender) { OnDllChange(Sender, 24, 2); }
void __fastcall TForm1::ARTCAMSMV413Click(TObject *Sender) { OnDllChange(Sender, 24, 3); }
void __fastcall TForm1::ARTCAMS800MIClick(TObject *Sender) { OnDllChange(Sender, 24, 4); }
void __fastcall TForm1::ARTCAMS036MIClick(TObject *Sender) { OnDllChange(Sender, 24, 5); }
void __fastcall TForm1::ARTCAMS150PClick (TObject *Sender) { OnDllChange(Sender, 24, 6); }


//---------------------------------------------------------------------------

void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
	m_CArtCamSdk[0].SnapShot((LPBYTE)m_BmpDraw[0]->ScanLine[m_BmpDraw[0]->Height-1], ArtCamGetSize(0), 0);
	m_CArtCamSdk[1].SnapShot((LPBYTE)m_BmpDraw[1]->ScanLine[m_BmpDraw[1]->Height-1], ArtCamGetSize(1), 0);

	// Draw image
	DrawImage();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormDestroy(TObject *Sender)
{
	UnhookWindowsHookEx(m_Hook);
	m_Hook = NULL;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormResize(TObject *Sender)
{
	// In Preview Mode, display size varies when size of child window is changed.
	if(0 == m_PreviewMode){
		HWND Child1 = GetWindow(Handle, GW_CHILD);
		HWND Child2 = GetWindow(Child1, GW_HWNDNEXT);
		RECT rc = GetClientRect();

		SetWindowPos(Child1, HWND_TOP, 0, 0, rc.right / 2, rc.bottom, SWP_NOMOVE | SWP_NOZORDER | SWP_SHOWWINDOW);
		SetWindowPos(Child2, HWND_TOP, rc.right / 2, 0, rc.right / 2, rc.bottom, SWP_NOZORDER | SWP_SHOWWINDOW);
	}
}
//---------------------------------------------------------------------------

